@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 #-------------------------------------------------------------------------------
 # Library modules import
@@ -6,8 +6,8 @@
 
 import sys
 
-if sys.version_info[:2] < (2,6):
-    sys.stderr.write("This script needs Python 2.6 at least\n")
+if sys.version_info[:2] < (3, 4):
+    sys.stderr.write("This script needs Python 3.4 at least\n")
 
 import platform
 
@@ -39,6 +39,7 @@ def run_command(cmd, stage, app, log):
 
     p = subprocess.Popen(cmd,
                          shell=True,
+                         universal_newlines=True,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT)
 
@@ -46,7 +47,7 @@ def run_command(cmd, stage, app, log):
     log.write(output[0])
 
     if p.returncode != 0:
-        sys.stderr.write("Error during " + string.lower(stage) +
+        sys.stderr.write("Error during " + stage.lower() +
                          " stage of " + app + ".\n")
         sys.stderr.write("See " + log.name + " for more information.\n")
         sys.exit(1)
@@ -65,6 +66,7 @@ def run_test(cmd):
 
     p = subprocess.Popen(cmd,
                          shell=True,
+                         universal_newlines=True,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.STDOUT)
 
@@ -87,11 +89,11 @@ def check_directory():
     top_srcdir, script_name = os.path.split(script_path)
     abscwd = os.path.abspath(os.getcwd())
 
-    if abscwd.find(top_srcdir) == 0:
+    if not os.path.relpath(abscwd, top_srcdir)[0:2] == '..':
         message = \
 """
 The '%(script_name)s' installer script should not be run from inside the
-'%(top_srcdir)s' Code_Saturne source directory,
+'%(top_srcdir)s' code_saturne source directory,
 but from a separate directory.
 
 We recommend running for example:
@@ -133,8 +135,9 @@ def find_executable(names, env_var=None):
     # test it first.
 
     if env_var:
-        if os.environ.has_key(env_var):
-            return os.environ(env_var)
+        k = os.environ.get(env_var)
+        if k:
+            return k
         else:
             for a in sys.argv[1:]:
                 if a.find(env_var) == 0:
@@ -173,7 +176,10 @@ class Package:
         self.version = version
         self.archive = archive
         if self.archive:
-            self.url = url % self.archive
+            try:
+                self.url = url % self.archive
+            except Exception:
+                self.url = url
         else:
             self.url = None
 
@@ -227,17 +233,17 @@ class Package:
 
     def download(self):
 
-        if sys.version_info[0] < (3):
-            import urllib2
-            u = urllib2.urlopen(self.url)
-            data = u.read()
-            f = open(self.archive, 'wb')
-            f.write(data)
-            f.close()
-
-        else:
-            import urllib.request
+        import urllib.request
+        try:
             urllib.request.urlretrieve(self.url, self.archive)
+        except Exception as e:
+            import traceback
+            exc_type, exc_value, exc_traceback = sys.exc_info()
+            traceback.print_exception(exc_type, exc_value, exc_traceback,
+                                      limit=2, file=sys.stderr)
+            print("")
+            print("Failed downloading " + str(self.url))
+            pass
 
     #---------------------------------------------------------------------------
 
@@ -262,6 +268,7 @@ class Package:
 
             p = subprocess.Popen('unzip ' + self.archive,
                                  shell=True,
+                                 universal_newlines=True,
                                  stdout=sys.stdout,
                                  stderr=sys.stderr)
             output = p.communicate()
@@ -281,7 +288,11 @@ class Package:
             tar = tarfile.open(self.archive)
 
             first_member = tar.next()
-            relative_source_dir = first_member.name.split(os.path.sep)[0]
+            path = first_member.name.split(os.path.sep)
+            if path[0] == '.':
+                relative_source_dir = path[1]
+            else:
+                relative_source_dir = path[0]
             self.source_dir = os.path.abspath(relative_source_dir)
 
             try:
@@ -363,111 +374,6 @@ class Package:
 
     #---------------------------------------------------------------------------
 
-    def install_ptscotch(self):
-
-        current_dir = os.getcwd()
-
-        build_dir = self.source_dir + '.build'
-        if os.path.isdir(build_dir): shutil.rmtree(build_dir)
-
-        # Create some install directories in case install script does not work
-        if self.install_dir:
-            inc_dir = os.path.join(self.install_dir, 'include')
-            lib_dir = os.path.join(self.install_dir, 'lib')
-            for dir in [inc_dir, lib_dir]:
-                if not os.path.isdir(dir):
-                    os.makedirs(dir)
-
-        # Copy source files in build directory as VPATH feature is unsupported
-        shutil.copytree(self.source_dir, build_dir)
-
-        os.chdir(os.path.join(build_dir, 'src'))
-
-        # Work around Ubuntu Metis build bug
-        ldflags_add = ''
-        try:
-            p = subprocess.Popen(self.cc + ' -Xlinker --help',
-                                 shell=True,
-                                 stdout=subprocess.PIPE,
-                                 stderr=subprocess.PIPE)
-            output = p.communicate()[0]
-            if output.find("--no-as-needed") > -1:
-                ldflags_add = ' -Wl,--no-as-needed\n'
-        except Exception:
-            pass
-
-        if self.shared:
-            fdr = open('Make.inc/Makefile.inc.x86-64_pc_linux2.shlib')
-        else:
-            fdr = open('Make.inc/Makefile.inc.x86-64_pc_linux2')
-        fd = open('Makefile.inc','w')
-
-        re_thread = re.compile('-DSCOTCH_PTHREAD')
-        re_intsize32 = re.compile('-DINTSIZE32')
-        re_intsize64 = re.compile('-DINTSIZE64')
-        re_idxsize64 = re.compile('-DIDXSIZE64')
-
-        for line in fdr:
-
-            if line[0:3] in ['CCS', 'CCP', 'CCD']:
-                i1 = line.find('=')
-                line = line[0:i1] + '= ' + self.cc + '\n'
-            line = re.sub(re_thread, '', line)
-            line = re.sub(re_intsize32, '', line)
-            line = re.sub(re_intsize64, '', line)
-            line = re.sub(re_idxsize64, '-DIDXSIZE64 -DINTSIZE64', line)
-            if ldflags_add and line[0:7] == 'LDFLAGS':
-                line = line[:-1] + ldflags_add
-
-            fd.write(line)
-
-        fdr.close()
-        fd.close()
-
-        # Build and install
-        for target in ['scotch', 'ptscotch']:
-            run_command("make "+target, "Compile", self.name, self.log_file)
-            run_command("make install prefix="+self.install_dir,
-                        "Install", self.name, self.log_file)
-            run_command("make clean", "Clean", self.name, self.log_file)
-
-        # End of installation
-        os.chdir(current_dir)
-
-    #---------------------------------------------------------------------------
-
-    def install_parmetis(self):
-
-        current_dir = os.getcwd()
-
-        build_dir = self.source_dir + '.build'
-        if os.path.isdir(build_dir): shutil.rmtree(build_dir)
-
-        # Copy source files in build directory as VPATH feature is unsupported
-        shutil.copytree(self.source_dir, build_dir)
-
-        for d in [os.path.join(build_dir, 'metis'), build_dir]:
-
-            os.chdir(d)
-
-            configure = "make config prefix=" + self.install_dir
-            configure += " cc=" + self.cc
-            if self.cxx:
-                configure += " cxx=" + self.cxx
-            if self.shared:
-                configure += " shared=1 "
-
-            # Install the package and clean build directory
-            run_command(configure, "Configure", self.name, self.log_file)
-            run_command("make", "Compile", self.name, self.log_file)
-            run_command("make install", "Install", self.name, self.log_file)
-            run_command("make clean", "Clean", self.name, self.log_file)
-
-        # End of installation
-        os.chdir(current_dir)
-
-    #---------------------------------------------------------------------------
-
     def test_library(self, executables=None, header=None, libname=None):
 
         libroot = None
@@ -522,7 +428,7 @@ class Package:
             self.install_dir = None
 
 #-------------------------------------------------------------------------------
-# Class definition for Code_Saturne setup
+# Class definition for code_saturne setup
 #-------------------------------------------------------------------------------
 
 class Setup:
@@ -533,7 +439,10 @@ class Setup:
         self.top_srcdir = os.path.abspath(os.path.dirname(sys.argv[0]))
 
         # Optional libraries
-        self.optlibs = ['hdf5', 'cgns', 'med', 'scotch', 'parmetis', 'libxml2']
+        self.optlibs = ['hdf5', 'cgns', 'med', 'scotch']
+
+        # Optional libraries configure could find in salome
+        self.salome_optlibs = ['hdf5', 'cgns', 'med']
 
         # Logging file
         self.log_file = sys.stdout
@@ -541,10 +450,7 @@ class Setup:
         # Download packages
         self.download = 'yes'
 
-        # Code_Saturne language (may be en/fr)
-        self.language = 'en'
-
-        # Code_Saturne installation with debugging symbols
+        # code_saturne installation with debugging symbols
         self.debug = 'no'
 
         # Installation with shared libraries (not modifiable yet)
@@ -579,15 +485,15 @@ class Setup:
         # Packages definition
         self.packages = {}
 
-        # Code_Saturne
+        # code_saturne
 
         self.packages['code_saturne'] = \
             Package(name="Code_Saturne",
-                    description="Code_Saturne CFD tool",
+                    description="code_saturne CFD tool",
                     package="code_saturne",
                     version=None,
                     archive=None,
-                    url="http://code-saturne.org")
+                    url="https://code-saturne.org")
 
         p = self.packages['code_saturne']
 
@@ -602,61 +508,39 @@ class Setup:
             Package(name="HDF5",
                     description="Hierarchical Data Format",
                     package="hdf5",
-                    version="1.8.16",
-                    archive="hdf5-1.8.16.tar.gz",
-                    url="http://www.hdfgroup.org/ftp/HDF5/releases/hdf5-1.8.16/src/%s")
+                    version="1.12.3",
+                    archive="hdf5-1.12.3.tar.bz2",
+                    url="https://support.hdfgroup.org/ftp/HDF5/releases/hdf5-1.12/hdf5-1.12.3/src/%s")
 
         p = self.packages['hdf5']
-        p.config_opts = "--enable-production"
+        p.config_opts = "--enable-build-mode=production"
 
         # CGNS library
 
         self.packages['cgns'] = \
             Package(name="CGNS",
                     description="CFD General Notation System",
-                    package="cgnslib",
-                    version="3.2.1",
-                    archive="cgnslib_3.2.1.tar.gz",
-                    url="http://sourceforge.net/projects/cgns/files/cgnslib_3.2/%s/download")
+                    package="cgns",
+                    version="4.4.0",
+                    archive="CGNS-4.4.0.tar.gz",
+                    url="https://github.com/CGNS/CGNS/archive/v4.4.0.tar.gz")
 
         p = self.packages['cgns']
         p.config_opts = "-DCGNS_ENABLE_64BIT=ON -DCGNS_ENABLE_SCOPING=ON"
 
         # MED library
+        # Note: alternative address: https://www.code-saturne.org/releases/external/med-5.0.0.tar.bz2
 
         self.packages['med'] = \
             Package(name="MED",
                     description="Model for Exchange of Data",
                     package="med",
-                    version="3.1.0",
-                    archive="med-3.1.0.tar.gz",
-                    url="http://files.salome-platform.org/Salome/other/%s")
+                    version="5.0.0",
+                    archive="med-5.0.0.tar.bz2",
+                    url="https://files.salome-platform.org/Salome/medfile/%s")
 
         p = self.packages['med']
-        p.config_opts = "--with-med_int=int --disable-fortran"
-
-        # Libxml2 library (possible mirror at "ftp://fr.rpmfind.net/pub/libxml/%s")
-
-        self.packages['libxml2'] = \
-            Package(name="libxml2",
-                    description="XML library",
-                    package="libxml2",
-                    version="2.9.2",
-                    archive="libxml2-sources-2.9.2.tar.gz",
-                    url="ftp://xmlsoft.org/libxml2/%s")
-
-        p = self.packages['libxml2']
-        p.config_opts = "--with-ftp=no --with-http=no"
-
-        # ParMETIS
-
-        self.packages['parmetis'] = \
-            Package(name="parmetis",
-                    description="ParMETIS",
-                    package="parmetis",
-                    version="4.0.3",
-                    archive="parmetis-4.0.3.tar.gz",
-                    url="http://glaros.dtc.umn.edu/gkhome/fetch/sw/parmetis/%s")
+        p.config_opts = "--with-med_int=long --disable-fortran --disable-python"
 
         # SCOTCH
 
@@ -664,20 +548,24 @@ class Setup:
             Package(name="scotch",
                     description="PT-Scotch",
                     package="scotch",
-                    version="6.0.4",
-                    archive="scotch_6.0.4.tar.gz",
-                    url="https://gforge.inria.fr/frs/download.php/34099/%s")
+                    version="7.0.4",
+                    archive="scotch_7.0.4.tar.gz",
+                    url="https://gitlab.inria.fr/scotch/scotch/-/archive/v7.0.4/%s")
+        p = self.packages['scotch']
+        p.config_opts = "-DBUILD_LIBSCOTCHMETIS=OFF"
+        if self.shared:
+            p.config_opts += " -DBUILD_SHARED_LIBS=ON"
 
     #---------------------------------------------------------------------------
 
     def setup_defaults(self):
 
-        self.cc = find_executable(['cc', 'gcc', 'icc', 'xlc'], 'CC')
-        self.fc = find_executable(['f95', 'gfortran', 'ifort'], 'FC')
-        self.cxx = find_executable(['c++', 'g++', 'icpc', 'xlc++'], 'CXX')
+        self.cc = find_executable(['cc', 'gcc', 'icx', 'icc', 'xlc', 'clang'], 'CC')
+        self.fc = find_executable(['f95', 'gfortran', 'ifx', 'ifort'], 'FC')
+        self.cxx = find_executable(['c++', 'g++', 'icpx', 'icpc', 'xlc++', 'clang++'], 'CXX')
         self.mpicc = find_executable(['mpicc', 'mpicc.openmpi', 'mpicc.mpich'])
         self.mpicxx = find_executable(['mpicxx', 'mpicxx.openmpi', 'mpicxx.mpich'])
-        self.python = find_executable(['python'], 'PYTHON')
+        self.python = find_executable(['python3'], 'PYTHON')
 
         # Architecture name
         self.arch = os.uname()[0] + '_' + os.uname()[4]
@@ -709,18 +597,12 @@ class Setup:
                        header='med.h',
                        libname='medC')
 
-        # Libxml2 library (possible mirror at "ftp://fr.rpmfind.net/pub/libxml/%s")
-        p = self.packages['libxml2']
-
-        p.test_library(header='libxml2/libxml/parser.h',
-                       libname='xml2')
-
         # Expand user variables
 
         p = self.packages['code_saturne']
         from os.path import expanduser
         home = expanduser("~")
-        self.prefix = os.path.join(home, 'Code_Saturne', p.version)
+        self.prefix = os.path.join(home, 'code_saturne', p.version)
 
     #---------------------------------------------------------------------------
 
@@ -736,7 +618,7 @@ class Setup:
 """
 
 Please edit the 'setup' file in the current directory
-to define your Code_Saturne setup options.
+to define your code_saturne setup options.
 
 You may then re-run '%(script_path)s'
 to start the installation.
@@ -757,7 +639,7 @@ to start the installation.
         # setup file reading
         #
         try:
-            setupFile = file('setup', mode='r')
+            setupFile = open('setup', mode='r')
         except IOError:
             sys.stderr.write('Error: opening setup file\n')
             sys.exit(1)
@@ -784,7 +666,6 @@ to start the installation.
                     if not list[1] in ['default', 'auto']:
                         self.prefix = list[1]
                 elif key == 'debug': self.debug = list[1]
-                elif key == 'language': self.language = list[1]
                 elif key == 'use_arch': self.use_arch = list[1]
                 elif key == 'arch':
                     self.arch = list[1]
@@ -881,14 +762,6 @@ Check the setup file and some utilities presence.
                              "Please check your setup file.\n\n")
             sys.exit(1)
 
-        # Testing language option
-        if self.language not in ['en', 'fr']:
-            sys.stderr.write("\n*** Aborting installation:\n"
-                             "\'language\' option in the setup file "
-                             "should be \'en\' or \'fr'.\n"
-                             "Please check your setup file.\n\n")
-            sys.exit(1)
-
         # Testing prefix directory
         if self.prefix and not os.path.isdir(self.prefix):
             try:
@@ -940,11 +813,12 @@ Check the setup file and some utilities presence.
                                  "Please check your setup file.\n\n")
             sys.exit(1)
         else:
-            cmd = python + " -c \'import sys; print(sys.version[:3])\'"
+            cmd = python + " -c \'import sys; print(\"%d.%d\" % (sys.version_info.major, sys.version_info.minor))\'"
             if verbose == 'yes':
                 sys.stdout.write("     Python version is ")
             p = subprocess.Popen(cmd,
                                  shell=True,
+                                 universal_newlines=True,
                                  stdout=subprocess.PIPE,
                                  stderr=subprocess.STDOUT)
 
@@ -953,6 +827,15 @@ Check the setup file and some utilities presence.
                 if p.returncode == 0:
                     sys.stdout.write(output[0])
 
+        # Looking for SALOME path provided by the user
+        if self.salome and not os.path.isdir(self.salome):
+            sys.stderr.write("\n*** Aborting installation:\n"
+                             "\'%s\' SALOME directory is provided in the setup "
+                             "file but is not present.\n"
+                             "Please check your setup file.\n\n"
+                             % self.salome)
+            sys.exit(1)
+
         # Checking libraries options
         for lib in self.optlibs:
             p = self.packages[lib]
@@ -971,7 +854,9 @@ Check the setup file and some utilities presence.
                                  % lib)
                 sys.exit(1)
             if p.installation == 'no' and p.use == 'yes':
-                if not os.path.isdir(p.install_dir):
+                use_salome = self.salome and lib in self.salome_optlibs and \
+                             p.install_dir == 'salome'
+                if not os.path.isdir(p.install_dir) and not use_salome:
                     sys.stderr.write("\n*** Aborting installation:\n"
                                      "\'%(path)s\' path is provided for "
                                      "\'%(lib)s\' in the setup "
@@ -980,21 +865,11 @@ Check the setup file and some utilities presence.
                                      % {'path':p.install_dir, 'lib':lib})
                     sys.exit(1)
 
-        # Looking for SALOME path probided by the user
-        if self.salome and not os.path.isdir(self.salome):
-            sys.stderr.write("\n*** Aborting installation:\n"
-                             "\'%s\' SALOME directory is provided in the setup "
-                             "file but is not present.\n"
-                             "Please check your setup file.\n\n"
-                             % self.salome)
-            sys.exit(1)
-
-
         # Looking for make utility
         ret = run_test("make")
         if ret != 0:
             sys.stderr.write("\n*** Aborting installation:\n"
-                             "\'make\' utility is mandatory for Code_Saturne "
+                             "\'make\' utility is mandatory for code_saturne "
                              "compilation.\n"
                              "Please install development tools.\n\n")
             sys.exit(1)
@@ -1020,9 +895,7 @@ Check the setup file and some utilities presence.
             # Compilers
             p.cc = self.cc
             p.cxx = self.cxx
-            if lib in ['scotch'] and self.mpicc:
-                p.cc = self.mpicc
-            elif lib in ['code_saturne', 'parmetis']:
+            if lib in ['code_saturne']:
                 if self.mpicc:
                     p.cc = self.mpicc
                 if self.mpicxx:
@@ -1041,8 +914,6 @@ Check the setup file and some utilities presence.
         cgns = self.packages['cgns']
         med= self.packages['med']
         scotch = self.packages['scotch']
-        parmetis = self.packages['parmetis']
-        libxml2 = self.packages['libxml2']
 
         # Disable GUI
 
@@ -1063,8 +934,8 @@ Check the setup file and some utilities presence.
             if hdf5.install_dir:
                 config_opts = config_opts + " --with-hdf5=" + hdf5.install_dir
                 med.config_opts += " --with-hdf5=" + hdf5.install_dir
-                cgns.config_opts += " -DHDF5_INCLUDE_PATH=" + hdf5.install_dir + "/include" \
-                    + " -DHDF5_LIBRARY=" + hdf5.install_dir + "/lib/libhdf5.so"
+                cgns.config_opts += " -DCMAKE_PREFIX_PATH=" + hdf5.install_dir \
+                    + " -DHDF5_INCLUDE_PATH=" + hdf5.install_dir + "/include"
 
         # CGNS
 
@@ -1082,13 +953,6 @@ Check the setup file and some utilities presence.
             if med.install_dir:
                 config_opts = config_opts + " --with-med=" + med.install_dir
 
-        # ParMetis
-
-        if parmetis.use == 'no':
-            config_opts = config_opts + " --without-metis"
-        else:
-            config_opts = config_opts + " --with-metis=" + parmetis.install_dir
-
         # PT-Scotch
 
         if scotch.use == 'no':
@@ -1096,15 +960,6 @@ Check the setup file and some utilities presence.
         else:
             config_opts = config_opts + " --with-scotch=" + scotch.install_dir
 
-        # Libxml2
-
-        if libxml2.use == 'no':
-            config_opts = config_opts + " --without-libxml2"
-        else:
-            if libxml2.install_dir:
-                config_opts = config_opts + \
-                    " --with-libxml2=" + libxml2.install_dir
-
         # Python
 
         if self.python:
@@ -1115,11 +970,6 @@ Check the setup file and some utilities presence.
         if self.salome:
             config_opts = config_opts + " --with-salome=" + self.salome
 
-        # Language
-
-        if self.language == 'fr':
-            config_opts = config_opts + " --enable-french"
-
         # Build type
 
         if self.shared:
@@ -1137,7 +987,7 @@ Check the setup file and some utilities presence.
             for lib in self.optlibs:
                 p = self.packages[lib]
                 if p.installation == 'yes':
-                    sys.stdout.write("Download of %s\n" % p.name)
+                    sys.stdout.write("Download of %s\n  (%s)\n" % (p.name, p.url))
                     p.download()
             self.download = 'no'
             self.write_setup()
@@ -1152,12 +1002,7 @@ Check the setup file and some utilities presence.
                 sys.stdout.write("Installation of %s\n" % p.name)
                 if verbose == 'yes':
                     p.info()
-                if lib == 'scotch':
-                    p.install_ptscotch()
-                elif lib == 'parmetis':
-                    p.install_parmetis()
-                else:
-                    p.install()
+                p.install()
                 p.installation = 'no'
                 self.write_setup()
                 if verbose == 'yes':
@@ -1182,11 +1027,11 @@ Check the setup file and some utilities presence.
         #
         # setup file update
         #
-        sf = file(os.path.join(os.getcwd(), "setup"), mode='w')
+        sf = open(os.path.join(os.getcwd(), "setup"), mode='w')
 
         setupMain = \
 """#========================================================
-# Setup file for Code_Saturne installation
+# Setup file for code_saturne installation
 #========================================================
 #
 #--------------------------------------------------------
@@ -1195,14 +1040,7 @@ Check the setup file and some utilities presence.
 download  %(download)s
 #
 #--------------------------------------------------------
-# Language
-#   default: "en" english
-#   others:  "fr" french
-#--------------------------------------------------------
-language  %(lang)s
-#
-#--------------------------------------------------------
-# Install Code_Saturne with debugging symbols
+# Install code_saturne with debugging symbols
 #--------------------------------------------------------
 debug     %(debug)s
 #
@@ -1261,7 +1099,7 @@ disable_frontend  %(disable_frontend)s
 # This is the path for the main SALOME directory,
 # not the application directory.
 #
-# If Code_Saturne is built with SALOME support,
+# If code_saturne is built with SALOME support,
 # running "code_saturne salome" will launch the
 # associated application, containing the CFDSTUDY module.
 #--------------------------------------------------------
@@ -1272,30 +1110,28 @@ salome    %(salome)s
 # ------------------
 #
 # MED / HDF5  For MED file format support
-#             (used by SALOME and by Gmsh)
+#             (used by SALOME and by Gmsh).
 #
 # CGNS / HDF5 For CGNS file support
-#             (used by many meshing tools)
+#             (used by many meshing tools).
 #
-# Scotch (includes PT-Scotch) and/or ParMetis
-# for parallel partitioning
+# Scotch (includes PT-Scotch) for parallel partitioning
 #
 #   For Linux workstations, HDF5, CGNS, and even MED
 # packages may be available through the package manager.
-# HDF5 is also often available on large systems such as
-# IBM Blue Gene or Cray XT/XE/XC/XK.
+# HDF5 is also often available on large systems.
+# When building with SALOME, the platform distribution's
+# packages may be used, by setting in the matching entry
+# 'yes' under the "Use" column and simply 'salome' under
+# the path column.
 #
 # Scotch and Pt-Scotch are available in some Linux
-# distributtions, but may be built with options
-# incompatible with non-threaded Code_Saturne runs.
+# distributions, but may be built with options
+# incompatible with non-threaded code_saturne runs.
 #
-#   To install CGNS or ParMetis, the CMake
+#   To install CGNS, the CMake
 # configuration/installation tool is required
-# (it is available in most Linux distributions)
-#
-#   Libxml2 is needed to read XML files output by the
-# Graphical User Interface. It is generally available
-# through the package manager.
+# (it is available in most Linux distributions).
 #--------------------------------------------------------
 #
 #  Name    Use   Install  Path
@@ -1332,7 +1168,7 @@ salome    %(salome)s
 
         sf.write(setupMain
                  % { 'download':self.download, 'prefix':prefix,
-                     'lang':self.language, 'debug':self.debug,
+                     'debug':self.debug,
                      'use_arch':self.use_arch, 'arch':arch,
                      'cc':cc, 'mpicc':mpicc,
                      'fc':fc,
@@ -1361,7 +1197,7 @@ if __name__ == "__main__":
     # --------
     welcome = \
         """
-        Installation of Code_Saturne
+        Installation of code_saturne
         ____________________________
 
 The process will take several minutes.
@@ -1370,10 +1206,10 @@ You can have a look at the log file meanwhile.
 
     finalize = \
 """
-Before using Code_Saturne, please update your environment with:
+Before using code_saturne, please update your environment with:
 
   cspath=%(cspath)s
-  alias code_saturne="$cspath/bin"
+  alias code_saturne="$cspath/code_saturne"
 
 The documentation should then be available through the commands:
   code_saturne info -g refcard
@@ -1387,7 +1223,7 @@ installation documentation, available using:
 
     thanks = \
 """
-Thank you for choosing Code_Saturne!
+Thank you for choosing code_saturne!
 
 """
 
