@@ -3,9 +3,9 @@
  *============================================================================*/
 
 /*
-  This file is part of Code_Saturne, a general-purpose CFD tool.
+  This file is part of code_saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2024 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -891,6 +891,11 @@ ecs_pre_ccm__lit_maillage(const char  *nom_fic_maillage,
 
   err = CCMIOOpenFile(NULL, nom_fic_maillage, kCCMIORead, &root);
 
+  if (err != kCCMIONoErr)
+    ecs_error(__FILE__, __LINE__, 0,
+              _("Error %d opening STAR-CCM+ file: %s"),
+              (int)err, nom_fic_maillage);
+
   CCMIOGetVersion(&err, root.node, &version);
 
   maj_ver =  version / 10000;
