@@ -6,9 +6,9 @@
  *============================================================================*/
 
 /*
-  This file is part of Code_Saturne, a general-purpose CFD tool.
+  This file is part of code_saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2025 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -31,10 +31,12 @@
  *  Local headers
  *----------------------------------------------------------------------------*/
 
-#include "cs_base.h"
-#include "cs_sles.h"
-#include "cs_sles_it.h"
-#include "cs_time_plot.h"
+#include "base/cs_base.h"
+#include "alge/cs_grid.h"
+#include "alge/cs_sles.h"
+#include "alge/cs_sles_it.h"
+#include "alge/cs_sles_pc.h"
+#include "base/cs_time_plot.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -48,6 +50,19 @@ BEGIN_C_DECLS
  * Type definitions
  *============================================================================*/
 
+/*----------------------------------------------------------------------------
+ * Multigrid types
+ *----------------------------------------------------------------------------*/
+
+typedef enum {
+
+  CS_MULTIGRID_V_CYCLE,          /*!< Use a V-cycle */
+  CS_MULTIGRID_K_CYCLE,          /*!< Use a K-cycle */
+  CS_MULTIGRID_K_CYCLE_HPC,      /*!< Use a K-cycle tuned for HPC systems */
+  CS_MULTIGRID_N_TYPES           /*!< Number of multigrid types */
+
+} cs_multigrid_type_t;
+
 /* Multigrid linear solver context (opaque) */
 
 typedef struct _cs_multigrid_t  cs_multigrid_t;
@@ -56,6 +71,10 @@ typedef struct _cs_multigrid_t  cs_multigrid_t;
  *  Global variables
  *============================================================================*/
 
+/* Names for multigrid types */
+
+extern const char *cs_multigrid_type_name[];
+
 /*=============================================================================
  * Public function prototypes
  *============================================================================*/
@@ -74,15 +93,12 @@ cs_multigrid_initialize(void);
 void
 cs_multigrid_finalize(void);
 
-/*----------------------------------------------------------------------------
- * Indicate if multigrid solver API is used for at least one system.
- *
- * returns:
- *   true if at least one system uses a multigrid solver, false otherwise
+/*----------------------------------------------------------------------------*
+ * Set maximum grid level which should run on device (i.e. GPU).
  *----------------------------------------------------------------------------*/
 
-bool
-cs_multigrid_needed(void);
+void
+cs_multigrid_set_max_grid_level_for_device(int  level);
 
 /*----------------------------------------------------------------------------
  * Define and associate a multigrid sparse linear system solver
@@ -102,29 +118,33 @@ cs_multigrid_needed(void);
  * If needed, cs_sles_find() may be used to obtain a pointer to the
  * matching cs_sles_t container.
  *
- * parameters:
- *   f_id <-- associated field id, or < 0
- *   name <-- associated name if f_id < 0, or NULL
+ * \param[in]  f_id     associated field id, or < 0
+ * \param[in]  name     associated name if f_id < 0, or NULL
+ * \param[in]  mg_type  type of multigrid algorithm to use
  *
  * \return  pointer to new multigrid info and context
  */
 /*----------------------------------------------------------------------------*/
 
 cs_multigrid_t *
-cs_multigrid_define(int          f_id,
-                    const char  *name);
+cs_multigrid_define(int                   f_id,
+                    const char           *name,
+                    cs_multigrid_type_t   mg_type);
 
-/*----------------------------------------------------------------------------
- * Create multigrid linear system solver info and context.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create multigrid linear system solver info and context.
  *
  * The multigrid variant is an ACM (Additive Corrective Multigrid) method.
  *
- * returns:
- *   pointer to new multigrid info and context
- *----------------------------------------------------------------------------*/
+ * \param[in]  mg_type  type of multigrid algorithm to use
+ *
+ * \return  pointer to new multigrid info and context
+ */
+/*----------------------------------------------------------------------------*/
 
 cs_multigrid_t *
-cs_multigrid_create(void);
+cs_multigrid_create(cs_multigrid_type_t  mg_type);
 
 /*----------------------------------------------------------------------------
  * Destroy multigrid linear system solver info and context.
@@ -158,13 +178,10 @@ cs_multigrid_copy(const void  *context);
  *
  * parameters:
  *   mg                     <-> pointer to multigrid info and context
- *   aggregation_limit      <-- maximum allowed fine cells per coarse cell
- *   coarsening_type        <-- coarsening type:
- *                              0: algebraic, natural face traversal;
- *                              1: algebraic, face traveral by criteria;
- *                              2: algebraic, Hilbert face traversal;
+ *   aggregation_limit      <-- maximum allowed fine rows per coarse cell
+ *   coarsening_type        <-- coarsening type; see cs_grid_coarsening_t
  *   n_max_levels           <-- maximum number of grid levels
- *   min_g_cells            <-- global number of cells on coarse grids
+ *   min_g_rows             <-- global number of rows on coarse grids
  *                              under which no coarsening occurs
  *   p0p1_relax             <-- p0/p1 relaxation_parameter
  *   postprocess_block_size <-- if > 0, postprocess coarsening
@@ -172,17 +189,43 @@ cs_multigrid_copy(const void  *context);
  *----------------------------------------------------------------------------*/
 
 void
-cs_multigrid_set_coarsening_options(cs_multigrid_t  *mg,
-                                    int              aggregation_limit,
-                                    int              coarsening_type,
-                                    int              n_max_levels,
-                                    cs_gnum_t        min_g_cells,
-                                    double           p0p1_relax,
-                                    int              postprocess_block_size);
+cs_multigrid_set_coarsening_options
+  (cs_multigrid_t       *mg,
+   int                   aggregation_limit,
+   cs_grid_coarsening_t  coarsening_type,
+   int                   n_max_levels,
+   cs_gnum_t             min_g_rows,
+   double                p0p1_relax,
+   int                   postprocess_block_size);
+
+/*----------------------------------------------------------------------------*/
+/*
+ * \brief Set specific multigrid coarsening parameters for fine grids.
+ *
+ * \param[in, out]  mg                    pointer to multigrid info and context
+ * \param[in]       f_settings_threshold  grids of this level or higher use
+ *                                        standard (coarse grid) settings.
+ * \param[in]       aggregation_limit     maximum allowed fine rows
+ *                                        per coarse row
+ * \param[in]       coarsening_type       coarsening type;
+ *                                        see \ref cs_grid_coarsening_t
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_multigrid_set_coarsening_options_fine_grid
+  (cs_multigrid_t       *mg,
+   int                   f_settings_threshold,
+   int                   aggregation_limit,
+   cs_grid_coarsening_t  coarsening_type);
 
 /*----------------------------------------------------------------------------
  * Set multigrid parameters for associated iterative solvers.
  *
+ * On a GPU, some parameters may be replaced by the closest GPU equivalents.
+ * For finer control, use \ref cs_multigrid_set_coarsening_options_d after
+ * calling this function to modify parameters for solvers running on device.
+ *
  * parameters:
  *   mg                     <-> pointer to multigrid info and context
  *   descent_smoother_type  <-- type of smoother for descent
@@ -220,6 +263,69 @@ cs_multigrid_set_solver_options(cs_multigrid_t     *mg,
                                 double              precision_mult_ascent,
                                 double              precision_mult_coarse);
 
+/*----------------------------------------------------------------------------*/
+/*
+ * \brief Set multigrid device solver parameters for associated
+ *        iterative solvers.
+ *
+ * \param[in, out]  mg                      pointer to multigrid info
+ *                                          and context
+ * \param[in]       descent_smoother_type   type of smoother for descent
+ * \param[in]       ascent_smoother_type    type of smoother for ascent
+ * \param[in]       coarse_solver_type      type of solver for coarsest grid
+ * \param[in]       n_max_iter_descent      maximum iterations
+ *                                          per descent smoothing
+ * \param[in]       n_max_iter_ascent       maximum iterations
+ *                                          per ascent smoothing
+ * \param[in]       n_max_iter_coarse       maximum iterations
+ *                                          per coarsest solution
+ * \param[in]       poly_degree_descent     preconditioning polynomial degree
+ *                                          for descent phases (0: diagonal)
+ * \param[in]       poly_degree_ascent      preconditioning polynomial degree
+ *                                          for ascent phases (0: diagonal)
+ * \param[in]       poly_degree_coarse      preconditioning polynomial degree
+ *                                          for coarse solver (0: diagonal)
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_multigrid_set_solver_options_d(cs_multigrid_t     *mg,
+                                  cs_sles_it_type_t   descent_smoother_type,
+                                  cs_sles_it_type_t   ascent_smoother_type,
+                                  cs_sles_it_type_t   coarse_solver_type,
+                                  int                 n_max_iter_descent,
+                                  int                 n_max_iter_ascent,
+                                  int                 n_max_iter_coarse,
+                                  int                 poly_degree_descent,
+                                  int                 poly_degree_ascent,
+                                  int                 poly_degree_coarse);
+
+/*----------------------------------------------------------------------------*/
+/*
+ * \brief Set the max. number of cycles for a multigrid
+ *
+ * \param[in, out]  mg              pointer to multigrid info and context
+ * \param[in]       n_max_cycles    maximum number of cycles
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_multigrid_set_max_cycles(cs_multigrid_t     *mg,
+                            int                 n_max_cycles);
+
+/*----------------------------------------------------------------------------*/
+/*
+ * \brief Indicate if a multigrid solver requires an MSR matrix input.
+ *
+ * \param[in]  mg  pointer to multigrid info and context
+ *
+ * \return   true if MSR is needed, false otherwise.
+ */
+/*----------------------------------------------------------------------------*/
+
+bool
+cs_multigrid_need_msr(const cs_multigrid_t  *mg);
+
 /*----------------------------------------------------------------------------
  * Setup multigrid sparse linear equation solver.
  *
@@ -237,6 +343,26 @@ cs_multigrid_setup(void               *context,
                    const cs_matrix_t  *a,
                    int                 verbosity);
 
+/*----------------------------------------------------------------------------
+ * Setup multigrid sparse linear equation solver with separate
+ * convection-diffusion matrixes
+ *
+ * parameters:
+ *   context   <-> pointer to multigrid info and context
+ *                 (actual type: cs_multigrid_t  *)
+ *   name      <-- pointer to name of linear system
+ *   a         <-- associated matrix
+ *   conv_diff <-- convection-diffusion mode
+ *   verbosity <-- associated verbosity
+ *----------------------------------------------------------------------------*/
+
+void
+cs_multigrid_setup_conv_diff(void               *context,
+                             const char         *name,
+                             const cs_matrix_t  *a,
+                             bool                conv_diff,
+                             int                 verbosity);
+
 /*----------------------------------------------------------------------------
  * Call multigrid sparse linear equation solver.
  *
@@ -246,12 +372,12 @@ cs_multigrid_setup(void               *context,
  *   name          <-- pointer to name of linear system
  *   a             <-- matrix
  *   verbosity     <-- associated verbosity
- *   rotation_mode <-- halo update option for rotational periodicity
  *   precision     <-- solver precision
- *   r_norm        <-- residue normalization
+ *   r_norm        <-- residual normalization
  *   n_iter        --> number of iterations
- *   residue       --> residue
+ *   residual      --> residual
  *   rhs           <-- right hand side
+ *   vx_ini        <-- initial solution (vx if nonzero, nullptr if zero)
  *   vx            <-> system solution
  *   aux_size      <-- number of elements in aux_vectors
  *   aux_vectors   --- optional working area (internal allocation if NULL)
@@ -265,12 +391,12 @@ cs_multigrid_solve(void                *context,
                    const char          *name,
                    const cs_matrix_t   *a,
                    int                  verbosity,
-                   cs_halo_rotation_t   rotation_mode,
                    double               precision,
                    double               r_norm,
                    int                 *n_iter,
-                   double              *residue,
+                   double              *residual,
                    const cs_real_t     *rhs,
+                   cs_real_t           *vx_ini,
                    cs_real_t           *vx,
                    size_t               aux_size,
                    void                *aux_vectors);
@@ -279,7 +405,7 @@ cs_multigrid_solve(void                *context,
  * Free iterative sparse linear equation solver setup context.
  *
  * Note that this function should free resolution-related data, such as
- * buffers and preconditioning but doesd not free the whole context,
+ * buffers and preconditioning but does not free the whole context,
  * as info used for logging (especially performance data) is maintained.
 
  * parameters:
@@ -303,6 +429,19 @@ void
 cs_multigrid_log(const void  *context,
                  cs_log_t     log_type);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create a multigrid preconditioner.
+ *
+ * \param[in]  mg_type  type of multigrid algorithm to use
+ *
+ * \return  pointer to newly created preconditioner object.
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_sles_pc_t *
+cs_multigrid_pc_create(cs_multigrid_type_t  mg_type);
+
 /*----------------------------------------------------------------------------
  * Error handler for multigrid sparse linear equation solver.
  *
@@ -311,22 +450,20 @@ cs_multigrid_log(const void  *context,
  * It does nothing in case the maximum iteration count is reached.
  *
  * parameters:
- *   context       <-> pointer to multigrid sparse linear system solver info
- *                     (actual type: cs_multigrid_t  *)
- *   name          <-- pointer to name of linear system
+ *   sles          <-> pointer to solver object
  *   state         <-- convergence status
  *   a             <-- matrix
- *   rotation_mode <-- halo update option for rotational periodicity
  *   rhs           <-- right hand side
  *   vx            <-> system solution
+ *
+ * returns:
+ *   false (do not attempt new solve)
  *----------------------------------------------------------------------------*/
 
-void
-cs_multigrid_error_post_and_abort(void                         *context,
+bool
+cs_multigrid_error_post_and_abort(cs_sles_t                    *sles,
                                   cs_sles_convergence_state_t   state,
-                                  const char                   *name,
                                   const cs_matrix_t            *a,
-                                  cs_halo_rotation_t            rotation_mode,
                                   const cs_real_t              *rhs,
                                   cs_real_t                    *vx);
 
@@ -345,6 +482,65 @@ cs_multigrid_set_plot_options(cs_multigrid_t  *mg,
                               const char      *base_name,
                               bool             use_iteration);
 
+/*----------------------------------------------------------------------------*/
+/*
+ * \brief Query the global multigrid parameters for parallel grid merging.
+ *
+ * \param[in]   mg                   pointer to multigrid info and context
+ * \param[out]  rank_stride          number of ranks over which merging
+ *                                   takes place, or NULL
+ * \param[out]  rows_mean_threshold  mean number of rows under which merging
+ *                                   should be applied, or NULL
+ * \param[out]  rows_glob_threshold  global number of rows under which
+ *                                   merging should be applied, or NULL
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_multigrid_get_merge_options(const cs_multigrid_t  *mg,
+                               int                   *rank_stride,
+                               int                   *rows_mean_threshold,
+                               cs_gnum_t             *rows_glob_threshold);
+
+/*----------------------------------------------------------------------------*/
+/*
+ * \brief Set global multigrid parameters for parallel grid merging behavior.
+ *
+ * \param[in, out]  mg                   pointer to multigrid info and context
+ * \param[in]       rank_stride          number of ranks over which merging
+ *                                       takes place
+ * \param[in]       rows_mean_threshold  mean number of rows under which
+ *                                       merging should be applied
+ * \param[in]       rows_glob_threshold  global number of rows under which
+ *                                       merging should be applied
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_multigrid_set_merge_options(cs_multigrid_t  *mg,
+                               int              rank_stride,
+                               int              rows_mean_threshold,
+                               cs_gnum_t        rows_glob_threshold);
+
+/*----------------------------------------------------------------------------*/
+/*
+ * \brief Return a pointer to a grid associated with a given multigrid
+ *        setup and level.
+ *
+ * If the multigrid hierarchy is not set up, or a level coarser than the
+ * coarsest level is requested, nullptr is returned.
+
+ * \param[in]  mg     pointer to multigrid info and context
+ * \param[in]  level  level of the requested grid (or -1 for coarsest)
+ *
+ * \return  pointer to grid of requested level (NULL id not present)
+ */
+/*----------------------------------------------------------------------------*/
+
+const cs_grid_t *
+cs_multigrid_get_grid(const cs_multigrid_t  *mg,
+                      int                    level);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
