@@ -6,9 +6,9 @@
  *============================================================================*/
 
 /*
-  This file is part of Code_Saturne, a general-purpose CFD tool.
+  This file is part of code_saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2025 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -31,11 +31,10 @@
  *  Local headers
  *----------------------------------------------------------------------------*/
 
-#include "cs_base.h"
-#include "cs_halo_perio.h"
-#include "cs_matrix.h"
-#include "cs_time_plot.h"
-#include "cs_sles.h"
+#include "base/cs_base.h"
+#include "alge/cs_matrix.h"
+#include "alge/cs_sles.h"
+#include "alge/cs_sles_pc.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -55,14 +54,30 @@ BEGIN_C_DECLS
 
 typedef enum {
 
-  CS_SLES_PCG,             /* Preconditionned conjugate gradient */
-  CS_SLES_JACOBI,          /* Jacobi */
-  CS_SLES_BICGSTAB,        /* Bi-conjugate gradient stabilized */
-  CS_SLES_BICGSTAB2,       /* Bi-conjugate gradient stabilized - 2*/
-  CS_SLES_GMRES,           /* Generalized minimal residual */
-  CS_SLES_P_GAUSS_SEIDEL,  /* Process-local Gauss-Seidel */
-  CS_SLES_PCR3,            /* 3-layer conjugate residual */
-  CS_SLES_N_IT_TYPES       /* Number of resolution algorithms */
+  CS_SLES_PCG,                 /*!< Preconditioned conjugate gradient */
+  CS_SLES_FCG,                 /*!< Preconditions flexible conjugate gradient,
+                                    described in \cite Notay:2015 */
+  CS_SLES_IPCG,                /*!< Preconditions inexact conjugate gradient */
+  CS_SLES_JACOBI,              /*!< Jacobi */
+  CS_SLES_BICGSTAB,            /*!< Preconditioned BiCGstab
+                                    (biconjugate gradient stabilized) */
+  CS_SLES_BICGSTAB2,           /*!< Preconditioned BiCGstab2 */
+  CS_SLES_GCR,                 /*!< Generalized conjugate residual  */
+  CS_SLES_GMRES,               /*!< Preconditioned GMRES
+                                    (generalized minimal residual) */
+  CS_SLES_P_GAUSS_SEIDEL,      /*!< Process-local Gauss-Seidel */
+  CS_SLES_P_SYM_GAUSS_SEIDEL,  /*!< Process-local symmetric Gauss-Seidel */
+  CS_SLES_PCR3,                /*!< 3-layer conjugate residual */
+  CS_SLES_USER_DEFINED,        /*!< User-defined iterative solver */
+
+  CS_SLES_N_IT_TYPES,          /*!< Number of resolution algorithms
+                                    excluding smoother only */
+
+  CS_SLES_TS_F_GAUSS_SEIDEL,   /*!< Truncated forward Gauss-Seidel smoother */
+  CS_SLES_TS_B_GAUSS_SEIDEL,   /*!< Truncated backward Gauss-Seidel smoother */
+
+  CS_SLES_N_SMOOTHER_TYPES     /*!< Number of resolution algorithms
+                                    including smoother only */
 
 } cs_sles_it_type_t;
 
@@ -70,14 +85,54 @@ typedef enum {
 
 typedef struct _cs_sles_it_t  cs_sles_it_t;
 
+/* Forward type declarations */
+
+typedef struct _cs_sles_it_convergence_t  cs_sles_it_convergence_t;
+
 /*============================================================================
  *  Global variables
  *============================================================================*/
 
-/* Short names for matrix types */
+/* Names for iterative solver types */
 
 extern const char *cs_sles_it_type_name[];
 
+/*=============================================================================
+ * User function prototypes
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------
+ * Solution of A.vx = Rhs using a user-defined iterative solver
+ *
+ * On entry, vx is considered initialized.
+ *
+ * parameters:
+ *   c               <-- pointer to solver context info
+ *   a               <-- matrix
+ *   diag_block_size <-- diagonal block size (unused here)
+ *   convergence     <-- convergence information structure
+ *   rhs             <-- right hand side
+ *   vx_ini          <-- initial system solution
+ *                       (vx if nonzero, nullptr if zero)
+ *   vx              <-> system solution
+ *   aux_size        <-- number of elements in aux_vectors (in bytes)
+ *   aux_vectors     --- optional working area (allocation otherwise)
+ *
+ * returns:
+ *   convergence state
+ *----------------------------------------------------------------------------*/
+
+cs_sles_convergence_state_t
+cs_user_sles_it_solver(cs_sles_it_t              *c,
+                       const cs_matrix_t         *a,
+                       cs_lnum_t                  diag_block_size,
+                       cs_sles_it_convergence_t  *convergence,
+                       const cs_real_t           *rhs,
+                       cs_real_t                 *vx_ini,
+                       cs_real_t                 *vx,
+                       size_t                     aux_size,
+                       void                      *aux_vectors);
+
 /*=============================================================================
  * Public function prototypes
  *============================================================================*/
@@ -190,12 +245,12 @@ cs_sles_it_setup(void               *context,
  *   name          <-- pointer to system name
  *   a             <-- matrix
  *   verbosity     <-- verbosity level
- *   rotation_mode <-- halo update option for rotational periodicity
  *   precision     <-- solver precision
- *   r_norm        <-- residue normalization
+ *   r_norm        <-- residual normalization
  *   n_iter        --> number of iterations
- *   residue       --> residue
+ *   residual      --> residual
  *   rhs           <-- right hand side
+ *   vx_ini        <-- initial solution (vx if nonzero, nullptr if zero)
  *   vx            <-> system solution
  *   aux_size      <-- number of elements in aux_vectors (in bytes)
  *   aux_vectors   --- optional working area (internal allocation if NULL)
@@ -209,13 +264,13 @@ cs_sles_it_solve(void                *context,
                  const char          *name,
                  const cs_matrix_t   *a,
                  int                  verbosity,
-                 cs_halo_rotation_t   rotation_mode,
                  double               precision,
                  double               r_norm,
                  int                 *n_iter,
-                 double              *residue,
+                 double              *residual,
                  const cs_real_t     *rhs,
                  cs_real_t           *vx,
+                 cs_real_t           *vx_ini,
                  size_t               aux_size,
                  void                *aux_vectors);
 
@@ -261,7 +316,7 @@ cs_sles_it_type_t
 cs_sles_it_get_type(const cs_sles_it_t  *context);
 
 /*----------------------------------------------------------------------------
- * Return the initial residue for the previous solve operation with a solver.
+ * Return the initial residual for the previous solve operation with a solver.
  *
  * This is useful for convergence tests when this solver is used as
  * a preconditioning smoother.
@@ -274,11 +329,64 @@ cs_sles_it_get_type(const cs_sles_it_t  *context);
  *   context <-- pointer to iterative solver info and context
  *
  * returns:
- *   initial residue from last call to \ref cs_sles_solve with this solver
+ *   initial residual from last call to \ref cs_sles_solve with this solver
  *----------------------------------------------------------------------------*/
 
 double
-cs_sles_it_get_last_initial_residue(const cs_sles_it_t  *context);
+cs_sles_it_get_last_initial_residual(const cs_sles_it_t  *context);
+
+/*----------------------------------------------------------------------------
+ * Return a preconditioner context for an iterative sparse linear
+ * equation solver.
+ *
+ * This allows modifying parameters of a non default (Jacobi or polynomial)
+ * preconditioner.
+ *
+ * parameters:
+ *   context <-- pointer to iterative solver info and context
+ *
+ * returns:
+ *   pointer to preconditoner context
+ *----------------------------------------------------------------------------*/
+
+cs_sles_pc_t  *
+cs_sles_it_get_pc(cs_sles_it_t  *context);
+
+/*----------------------------------------------------------------------------
+ * \brief Assign a preconditioner to an iterative sparse linear equation
+ *        solver, transfering its ownership to the solver context.
+ *
+ * This allows assigning a non default (Jacobi or polynomial) preconditioner.
+ *
+ * The input pointer is set to NULL to make it clear the caller does not
+ * own the preconditioner anymore, though the context can be accessed using
+ * \ref cs_sles_it_get_pc.
+ *
+ * \param[in, out]  context   pointer to iterative solver info and context
+ * \param[in, out]  pc        pointer to preconditoner context
+ *----------------------------------------------------------------------------*/
+
+void
+cs_sles_it_transfer_pc(cs_sles_it_t     *context,
+                       cs_sles_pc_t    **pc);
+
+/*----------------------------------------------------------------------------
+ * Copy options from one iterative sparse linear system solver info
+ * and context to another.
+ *
+ * Optional plotting contexts are shared between the source and destination
+ * contexts.
+ *
+ * Preconditioner settings are to be handled separately.
+ *
+ * parameters:
+ *   src  <-- pointer to source info and context
+ *   dest <-> pointer to destination info and context
+ *----------------------------------------------------------------------------*/
+
+void
+cs_sles_it_transfer_parameters(const cs_sles_it_t  *src,
+                               cs_sles_it_t        *dest);
 
 /*----------------------------------------------------------------------------
  * Associate a similar info and context object with which some setup
@@ -309,17 +417,24 @@ cs_sles_it_set_shareable(cs_sles_it_t        *context,
 
 #if defined(HAVE_MPI)
 
-/*----------------------------------------------------------------------------
- * Set MPI communicator for dot products.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set MPI communicator for global reductions.
  *
- * parameters:
- *   context <-> pointer to iterative sparse linear system solver info
- *   comm    <-- MPI communicator
- *----------------------------------------------------------------------------*/
+ * The system is solved only on ranks with a non-null communicator or
+ * if the caller communicator has less than 2 ranks. convergence info
+ * is broadcast across the caller communicator.
+ *
+ * \param[in, out]  context      pointer to iterative solver info and context
+ * \param[in]       comm         MPI communicator for solving
+ * \param[in]       caller_comm  MPI communicator of caller
+ */
+/*----------------------------------------------------------------------------*/
 
 void
 cs_sles_it_set_mpi_reduce_comm(cs_sles_it_t  *context,
-                               MPI_Comm       comm);
+                               MPI_Comm       comm,
+                               MPI_Comm       caller_comm);
 
 #endif /* defined(HAVE_MPI) */
 
@@ -340,19 +455,80 @@ void
 cs_sles_it_assign_order(cs_sles_it_t   *context,
                         cs_lnum_t     **order);
 
-/*----------------------------------------------------------------------------
- * Assign symmetric option to iterative solver.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Retrieve the threshold value under which a breakdown happens in
+ *        solvers like BiCGStab or BiCGStab2
  *
- * This is useful only for Process-local Gauss-Seidel.
+ * \return the value of the threshold
+ */
+/*----------------------------------------------------------------------------*/
+
+double
+cs_sles_it_get_breakdown_threshold(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define the threshold value under which a breakdown happens in
+ *        solvers like BiCGStab or BiCGStab2
  *
- * parameters:
- *   context   <-> pointer to iterative solver info and context
- *   symmetric <-- symmetric if true
- *----------------------------------------------------------------------------*/
+ * \param[in]  threshold  value of the threshold
+ */
+/*----------------------------------------------------------------------------*/
 
 void
-cs_sles_it_set_symmetric(cs_sles_it_t   *context,
-                         bool            symmetric);
+cs_sles_it_set_breakdown_threshold(double  threshold);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define convergence level under which the fallback to another
+ *        solver may be used if applicable.
+ *
+ * Currently, this mechanism is used by default for solvers which may exhibit
+ * breakdown, such as BiCGstab and 3-layer conjugate residual solvers, which
+ * may fall back to a more robust preconditioned GMRES solver.
+ *
+ * For those solvers, the default threshold is \ref CS_SLES_MAX_ITERATION,
+ * meaning that reaching breakdown will lead to the use of the
+ * fallback mechanism.
+ *
+ * \param[in, out] context     pointer to iterative solver info and context
+ * \param[in]      threshold   convergence level under which fallback is used
+ * \param[in]      n_iter_max  maximum number of iterations fo fallback solver
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sles_it_set_fallback_threshold(cs_sles_it_t                 *context,
+                                  cs_sles_convergence_state_t   threshold,
+                                  int                           n_iter_max);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define the number of iterations to be done before restarting the
+ *        solver. Useful only for GCR or GMRES algorithms.
+ *
+ * \param[in, out]  context   pointer to iterative solver info and context
+ * \param[in]       interval  convergence level under which fallback is used
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sles_it_set_restart_interval(cs_sles_it_t  *context,
+                                int            interval);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define the max. number of iterations before stopping the algorithm
+ *
+ * \param[in, out]  context     pointer to iterative solver info and context
+ * \param[in]       n_max_iter  max. number of iterations
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sles_it_set_n_max_iter(cs_sles_it_t  *context,
+                          int            n_max_iter);
 
 /*----------------------------------------------------------------------------
  * Query mean number of rows under which Conjugate Gradient algorithm
@@ -409,23 +585,20 @@ cs_sles_it_log_parallel_options(void);
  * It does nothing in case the maximum iteration count is reached.
  *
  * parameters:
- *   context       <-> pointer to iterative sparse linear system solver info
- *                     (actual type: cs_sles_it_t  *)
+ *   sles          <-> pointer to solver object
  *   state         <-- convergence state
- *   name          <-- pointer to name of linear system
  *   a             <-- matrix
- *   rotation_mode <-- halo update option for rotational periodicity
  *   rhs           <-- right hand side
  *   vx            <-> system solution
- */
-/*----------------------------------------------------------------------------*/
+ *
+ * returns:
+ *   false (do not attempt new solve)
+ *----------------------------------------------------------------------------*/
 
-void
-cs_sles_it_error_post_and_abort(void                         *context,
+bool
+cs_sles_it_error_post_and_abort(cs_sles_t                    *sles,
                                 cs_sles_convergence_state_t   state,
-                                const char                   *name,
                                 const cs_matrix_t            *a,
-                                cs_halo_rotation_t            rotation_mode,
                                 const cs_real_t              *rhs,
                                 cs_real_t                    *vx);
 
@@ -445,21 +618,23 @@ cs_sles_it_set_plot_options(cs_sles_it_t  *context,
                             bool           use_iteration);
 
 /*----------------------------------------------------------------------------
- * Assign existing time plot to iterative sparse linear equation solver.
- *
- * This is useful mainly when a time plot has a longer lifecycle than
- * the linear solver context, such as inside a multigrid solver.
+ * Convergence test.
  *
  * parameters:
- *   context    <-> pointer to iterative solver info and context
- *   time_plot  <-- pointer to time plot structure
- *   time_stamp <-- associated time stamp
+ *   c           <-- pointer to solver context info
+ *   n_iter      <-- Number of iterations done
+ *   residual    <-- Non normalized residual
+ *   convergence <-> Convergence information structure
+ *
+ * returns:
+ *   convergence status.
  *----------------------------------------------------------------------------*/
 
-void
-cs_sles_it_assign_plot(cs_sles_it_t    *context,
-                       cs_time_plot_t  *time_plot,
-                       int              time_stamp);
+cs_sles_convergence_state_t
+cs_sles_it_convergence_test(cs_sles_it_t              *c,
+                            unsigned                   n_iter,
+                            double                     residual,
+                            cs_sles_it_convergence_t  *convergence);
 
 /*----------------------------------------------------------------------------*/
 
