@@ -1,8 +1,8 @@
 !-------------------------------------------------------------------------------
 
-! This file is part of Code_Saturne, a general-purpose CFD tool.
+! This file is part of code_saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2024 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,7 +26,7 @@
 
 !> \file atimbr.f90
 !>
-!> \brief Atmospheric Imbrication module
+!> \brief Atmospheric Imbrication module.
 !>  This module contains the data structure and subroutines to
 !>  perform atmospheric imbrication or nesting of a CFD domain within
 !>  a large scale meteorological field.
@@ -34,42 +34,44 @@
 !>  format of meteo files) an interpolation is performed for each
 !>  boundary face both spatially and temporally (using Cressman method)
 !>
-!>  This imbrication reads additional meteo files
+!>  This imbrication reads additional meteo files.
 !______________________________________________________________________________
 
 module atimbr
+!> \defgroup at_imbrication
 
 use entsor
 use atincl
 implicit none
 
+!> \addtogroup at_imbrication
+!> \{
 ! --------------------------------------------------------------
-! activation flag
+!> activation flag
 ! --------------------------------------------------------------
-logical imbrication_flag
-logical imbrication_verbose
-save imbrication_verbose
+logical(c_bool), pointer :: imbrication_flag
+logical(c_bool), pointer, save :: imbrication_verbose
 
 ! --------------------------------------------------------------
-! flags for activating the cressman interpolation for the boundary
-! conditions
+!> Flags for activating the cressman interpolation for the boundary
+!> conditions
 ! --------------------------------------------------------------
-logical cressman_u
-logical cressman_v
-logical cressman_tke
-logical cressman_eps
-logical cressman_theta
-logical cressman_qw
-logical cressman_nc
+logical(c_bool), pointer :: cressman_u
+logical(c_bool), pointer :: cressman_v
+logical(c_bool), pointer :: cressman_tke
+logical(c_bool), pointer :: cressman_eps
+logical(c_bool), pointer :: cressman_theta
+logical(c_bool), pointer :: cressman_qw
+logical(c_bool), pointer :: cressman_nc
 
 ! --------------------------------------------------------------
-! numerical parameters for the cressman interpolation formulas
+!> numerical parameters for the cressman interpolation formulas
 ! --------------------------------------------------------------
-double precision :: horizontal_influence_radius
-double precision :: vertical_influence_radius
+real(c_double), pointer :: vertical_influence_radius
+real(c_double), pointer :: horizontal_influence_radius
 
 ! --------------------------------------------------------------
-!
+!> Parameter for "meteo" files
 ! --------------------------------------------------------------
 integer line_len
 parameter(line_len = 132)
@@ -78,31 +80,34 @@ character(line_len), dimension(:), allocatable :: imbrication_files
 integer number_of_files
 character*(3) skip_chars
 parameter (skip_chars = "/#!")
-
+!> Profile dimension variable
 integer thermal_profile_dim
 integer dynamical_profile_dim
+!> Time sections per files
 integer sections_per_file
 data thermal_profile_dim /-1/
 data dynamical_profile_dim /-1/
 data sections_per_file /-1/
 
 ! --------------------------------------------------------------
-! read data
+!> read data from "meteo" files
 ! --------------------------------------------------------------
+!> Time variables
 integer, dimension(:,:), allocatable :: years
 integer, dimension(:,:), allocatable :: ordinals
 integer, dimension(:,:), allocatable :: hours
 integer, dimension(:,:), allocatable :: minutes
 double precision, dimension(:,:),allocatable :: seconds
-
+!> Positions
 double precision, dimension(:,:), allocatable :: xpos
 double precision, dimension(:,:), allocatable :: ypos
 double precision, dimension(:,:), allocatable :: ground_pressure
-
+!> Vertical grid for temperature and humidity variables
 double precision, dimension(:,:,:), allocatable :: zt
 double precision, dimension(:,:,:), allocatable :: tempC
 double precision, dimension(:,:,:), allocatable :: qw
 double precision, dimension(:,:,:), allocatable :: Nc
+!> Vertical grid for wind variables
 double precision, dimension(:,:,:), allocatable :: zd
 double precision, dimension(:,:,:), allocatable :: u
 double precision, dimension(:,:,:), allocatable :: v
@@ -110,7 +115,7 @@ double precision, dimension(:,:,:), allocatable :: tke
 double precision, dimension(:,:,:), allocatable :: eps
 
 ! --------------------------------------------------------------
-! derived data
+!> derived data
 ! --------------------------------------------------------------
 double precision, dimension(:,:), allocatable,target :: times
 double precision, dimension(:,:,:), allocatable :: pressure
@@ -118,7 +123,7 @@ double precision, dimension(:,:,:), allocatable :: theta
 double precision, dimension(:,:,:), allocatable :: density
 
 ! --------------------------------------------------------------
-! time interpolated profiles
+!> time interpolated profiles
 ! --------------------------------------------------------------
 double precision, dimension(:,:), allocatable :: ti_zt
 double precision, dimension(:,:), allocatable :: ti_tempC
@@ -134,28 +139,94 @@ double precision, dimension(:,:), allocatable :: ti_theta
 double precision, dimension(:,:), allocatable :: ti_density
 
 ! --------------------------------------------------------------
-! red tape for mesmap
+!> additional variables
 ! --------------------------------------------------------------
 double precision, dimension(:,:,:), allocatable :: coordinates_th
 double precision, dimension(:,:,:), allocatable :: influence_param_th
 double precision, dimension(:,:,:), allocatable :: coordinates_dyn
 double precision, dimension(:,:,:), allocatable :: influence_param_dyn
-integer id_u
-integer id_v
-integer id_tke
-integer id_eps
-integer id_theta
-integer id_qw
-integer id_nc
+integer(c_int), pointer ::  id_u
+integer(c_int), pointer ::  id_v
+integer(c_int), pointer ::  id_qw
+integer(c_int), pointer ::  id_nc
+integer(c_int), pointer ::  id_tke
+integer(c_int), pointer ::  id_eps
+integer(c_int), pointer ::  id_theta
+
+interface
+
+  subroutine cs_f_atmo_get_pointers_imbrication(p_imbrication_flag,       &
+    p_imbrication_verbose, p_cressman_u, p_cressman_v, p_cressman_qw,     &
+    p_cressman_nc, p_cressman_tke, p_cressman_eps, p_cressman_theta,      &
+    p_vertical_influence_radius, p_horizontal_influence_radius,           &
+    p_id_u, p_id_v, p_id_nc, p_id_qw, p_id_tke, p_id_eps, p_id_theta)     &
+    bind(C, name='cs_f_atmo_get_pointers_imbrication')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      type(c_ptr), intent(out) :: p_imbrication_flag, p_cressman_qw
+      type(c_ptr), intent(out) :: p_imbrication_verbose, p_cressman_u
+      type(c_ptr), intent(out) :: p_horizontal_influence_radius, p_cressman_v
+      type(c_ptr), intent(out) :: p_cressman_nc, p_cressman_tke, p_cressman_eps
+      type(c_ptr), intent(out) :: p_vertical_influence_radius, p_cressman_theta
+      type(c_ptr), intent(out) :: p_id_u, p_id_v, p_id_nc, p_id_qw, p_id_tke
+      type(c_ptr), intent(out) :: p_id_eps, p_id_theta
+    end subroutine cs_f_atmo_get_pointers_imbrication
+
+end interface
 
 ! --------------------------------------------------------------
-! 1D array of times at which profiles are given
+!> 1D array of times at which profiles are given
 ! --------------------------------------------------------------
 double precision, dimension(:), pointer :: times_sequence=>null()
+!> \}
 contains
 
 ! ----------------------------------------------------------------
-!
+!> \brief  Map Fortran to C variables
+! ----------------------------------------------------------------
+
+subroutine atmo_init_imbrication()
+  implicit none
+  type(c_ptr) :: c_imbrication_flag, c_cressman_qw
+  type(c_ptr) :: c_imbrication_verbose, c_cressman_u
+  type(c_ptr) :: c_horizontal_influence_radius, c_cressman_v
+  type(c_ptr) :: c_cressman_nc, c_cressman_tke, c_cressman_eps
+  type(c_ptr) :: c_vertical_influence_radius, c_cressman_theta
+  type(c_ptr) :: c_id_u, c_id_v, c_id_qw, c_id_nc, c_id_tke, c_id_eps, c_id_theta
+
+  call cs_f_atmo_get_pointers_imbrication(c_imbrication_flag,                &
+       c_imbrication_verbose, c_cressman_u, c_cressman_v, c_cressman_qw,     &
+       c_cressman_nc, c_cressman_tke, c_cressman_eps, c_cressman_theta,      &
+       c_vertical_influence_radius, c_horizontal_influence_radius,           &
+       c_id_u, c_id_v, c_id_qw,c_id_nc, c_id_tke, c_id_eps, c_id_theta)
+
+  call c_f_pointer(c_imbrication_flag, imbrication_flag)
+  call c_f_pointer(c_imbrication_verbose, imbrication_verbose)
+
+  call c_f_pointer(c_cressman_u, cressman_u)
+  call c_f_pointer(c_cressman_v, cressman_v)
+  call c_f_pointer(c_cressman_qw, cressman_qw)
+  call c_f_pointer(c_cressman_nc, cressman_nc)
+  call c_f_pointer(c_cressman_tke, cressman_tke)
+  call c_f_pointer(c_cressman_eps, cressman_eps)
+  call c_f_pointer(c_cressman_theta, cressman_theta)
+
+  call c_f_pointer(c_vertical_influence_radius, vertical_influence_radius)
+  call c_f_pointer(c_horizontal_influence_radius, horizontal_influence_radius)
+
+  call c_f_pointer(c_id_u, id_u)
+  call c_f_pointer(c_id_v, id_v)
+  call c_f_pointer(c_id_qw, id_qw)
+  call c_f_pointer(c_id_nc, id_nc)
+  call c_f_pointer(c_id_tke, id_tke)
+  call c_f_pointer(c_id_eps, id_eps)
+  call c_f_pointer(c_id_theta, id_theta)
+
+end subroutine atmo_init_imbrication
+
+! ----------------------------------------------------------------
+!> \brief Allocate variables adapted to the number of files
+!>  and time step to be considered
 ! ----------------------------------------------------------------
 subroutine allocate_all()
 implicit none
@@ -197,9 +268,11 @@ allocate(eps(dynamical_profile_dim,sections_per_file,number_of_files))
 end subroutine allocate_all
 
 ! ----------------------------------------------------------------
-!
+!> \brief  Final step for deallocation
 ! ----------------------------------------------------------------
-subroutine finalize_imbrication()
+subroutine finalize_imbrication() &
+  bind(C, name='cs_f_finalize_imbrication')
+
 implicit none
 ! --- section of explicit deallocations -----------------
 deallocate(imbrication_files)
@@ -251,8 +324,14 @@ deallocate(influence_param_dyn)
 end subroutine finalize_imbrication
 
 ! ---------------------------------------------------------------------------
-! -
-! ---------------------------------------------------------------------------
+!> \brief Time interpolation of all profiles -
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   the_time    current time
+!-------------------------------------------------------------------------------
 subroutine interpolate_all_profiles(the_time)
 implicit none
 double precision the_time
@@ -512,6 +591,9 @@ endif
 
 end subroutine interpolate_all_profiles
 
+! ----------------------------------------------------------------
+!> \brief Print the interpolated profiles for checking purposes
+! ----------------------------------------------------------------
 subroutine dump_interpolated_profiles
 implicit none
 integer lb, ub
@@ -690,50 +772,63 @@ end subroutine dump_interpolated_profiles
 
 
 ! ----------------------------------------------------------------
-!
+!> \brief  Converts a (year,ordinal) date to julian calendar date
+!> for calculating time shifts
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   year
+!> \param[in]   ordinal     number of the day in the year
+!>                          e.g 1st january has ordinal 1
+!>                               31 december 365 or 366
 ! ----------------------------------------------------------------
 integer function yo2j(year,ordinal)
+!-------------------------------------------------------------------------------
 !
-! converts a year,ordinal day in julian day for calculating time shifts
-! ordinal is the number of the day in the year: e.g 1st january has ordinal 1
-!                                                   31 december 365 or 366
 implicit none
 integer year,ordinal
-! I am puzzled by the (1-14)/12 : why not writing -1?
-yo2j= ordinal + ((1461 * (year + 4800 + (1 - 14) / 12)) / 4 +   &
-        (367 * (1 - 2 - 12 * ((1 - 14) / 12))) / 12 -           &
-        (3 * ((year + 4900 + (1 - 14) / 12) / 100)) / 4         &
-        + 1 - 32075) - 1
+! I am puzzled by the (1-14)/12 : why not write -1?
+
+! yo2j= ordinal + ((1461 * (year + 4800 + (1 - 14) / 12)) / 4 +        &
+!                  (367 * (1 - 2 - 12 * ((1 - 14) / 12))) / 12 -       &
+!                  (3 * ((year + 4900 + (1 - 14) / 12) / 100)) / 4     &
+!                   + 1 - 32075) - 1
+
+yo2j= ordinal + ((1461 * (year + 4800)) / 4     &
+      -30 - (3 * ((year + 4900) / 100)) / 4     &
+      + 1 - 32075) - 1
 end function yo2j
 
 
-! ***************************************************************
-!
-! ***************************************************************
+! ----------------------------------------------------------------
+!> \brief Reads a file having in each significative line a file name
+!>  it returns then as 'the_list' the list of lines read
+!>  a line is significative if it's first char is not / or # or !
+!>  The following 3 lines give an example from which one must remove the
+!>  first two characters.
+!>- /list of files
+!>- profile_one.txt
+!>- profile_two.txt
+!>
+!> Beware that blank lines or lines starting with blanks+ comment_char are
+!> NOT ignored.
+! ----------------------------------------------------------------
+! Arguments
+! ----------------------------------------------------------------
+!> \param[in]   a_file      the file with list of file names
+!> \param[out]  the_list    the list of file names
+! ----------------------------------------------------------------
 subroutine read_files_list(a_file,the_list)
-! ----------------------------------------------------------------
-!
-! ----------------------------------------------------------------
-! reads a file having in each significative line a file name
-! it returns then as 'the_list' the list of lines read
-! a line is significative if it's first char is not / or # or !
-! The following 3 lines give an example from which one must remove the
-! first two characters.
-! /list of files
-! profile_one.txt
-! profile_two.txt
-!
-! Beware that blank lines or lines starting with blanks+ comment_char are
-! NOT ignored.
 
+implicit none
 ! ----------------------------------------------------------------
 ! declarations
 ! ----------------------------------------------------------------
-implicit none
 character(line_len) :: a_file
 character(line_len), dimension(:), allocatable :: the_list
 integer unilog
-parameter (unilog = 10)
 character(line_len) :: current_line
 integer l_iostat, l_stat
 integer counter
@@ -741,7 +836,7 @@ integer counter
 ! ----------------------------------------------------------------
 ! executables
 ! ----------------------------------------------------------------
-open(unilog,file=imbrication_files_list,status='old', &
+open(newunit=unilog,file=imbrication_files_list,status='old', &
      form='formatted',iostat=l_iostat)
 
 counter = 0
@@ -754,7 +849,7 @@ number_of_files = counter
 
 allocate(the_list(number_of_files),stat=l_stat)
 ! find_next closes the file when reaching the end
-open(unilog,file=imbrication_files_list,status='old',&
+open(newunit=unilog,file=imbrication_files_list,status='old',&
      form='formatted',iostat=l_iostat)
 counter = 0
 do while (.true.)
@@ -767,8 +862,17 @@ end subroutine read_files_list
 
 
 ! ----------------------------------------------------------------
-!
-! ----------------------------------------------------------------
+!> \brief  Find next validated line
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   unilog          logical unit number of the reading file
+!> \param[out]  current_line    the characters of the line
+!> \param[in]   meteo_file      name of the 'meteo' file
+!> \param[out]  l_iostat        logical status of I/O following a read statement
+!-------------------------------------------------------------------------------
 subroutine find_next_line(unilog,current_line,meteo_file,l_iostat)
 implicit none
 integer unilog
@@ -799,35 +903,38 @@ end subroutine find_next_line
 
 
 ! ----------------------------------------------------------------
-!
-! ----------------------------------------------------------------
+!> \brief Reads a meteo_file for code_saturne Atmospheric Physics option
+!>
+!> They contain an arbitrary number (>=1) of sections having the following
+!> structure.
+!> Comment lines start with a slash / as first character
+!>
+!>- yyyy,dd,hh,mm,ss
+!>- xpos,ypos
+!>- ground pressure
+!>- nt (thermal profile dimension)
+!>- nt lines of
+!>- zt,tempC,qw(kg/kg),Ndrops(1/cm3)
+!>- nd (thermal profile dimension)
+!>- nd lines of
+!>- zd,u,v,k,eps
+!>
+!> WARNINGS:
+!> Beware that all dimensions nt,nd must be the same as the first one.
+!>
+!> Beware that blank lines or lines starting with blanks+ comment_char are
+!> NOT ignored.
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   meteo_file      "meteo" file name
+!-------------------------------------------------------------------------------
 subroutine read_meteo_file(meteo_file)
 !
-! reads a meteo_file for Code_Saturne Atmospheric Physics option
-!
-! They contain an arbitrary number>=1 of sections having the following
-! structure.
-! Comment lines start with a slash / as first character
-!
-! yyyy,dd,hh,mm,ss
-! xpos,ypos
-! ground pressure
-! nt (thermal profile dimension)
-! nt lines of
-! zt,tempC,qw(kg/kg),Ndrops(1/cm3)
-! nd (thermal profile dimension)
-! nd lines of
-! zd,u,v,k,eps
-!
-! WARNINGS:
-! Beware that all dimensions nt,nd must be the same as the first one.
-!
-! Beware that blank lines or lines starting with blanks+ comment_char are
-! NOT ignored.
-!
 character(line_len) :: meteo_file
 integer unilog
-parameter (unilog = 10)
 character(line_len) :: current_line
 integer l_iostat
 integer first, last
@@ -847,7 +954,7 @@ integer i
 ! ----------------------------------------------------------------
 file_count = file_count + 1
 
-open(unilog,file=meteo_file,status='old',form='formatted',iostat=l_iostat)
+open(newunit=unilog,file=meteo_file,status='old',form='formatted',iostat=l_iostat)
 count_lines = 0
 
 ! --------------------------------------------------------------
@@ -953,7 +1060,7 @@ endif
 call bounds(meteo_file,len(meteo_file),first,last)
 if (imbrication_verbose) write(nfecra,*) "reopening the file'", &
      meteo_file(first:last),"'"
-open(unilog,file=meteo_file,status='old',form='formatted',iostat=l_iostat)
+open(newunit=unilog,file=meteo_file,status='old',form='formatted',iostat=l_iostat)
 if (l_iostat.ne.0) then
   call bounds(meteo_file,len(meteo_file),first,last)
   write(nfecra,*) "read_meteo_file could not open file '", &
@@ -1079,7 +1186,7 @@ end subroutine read_meteo_file
 
 
 ! *****************************************************************************
-!
+!> \brief    Checks the time variables to ensure the chronology
 ! *****************************************************************************
 subroutine check_chronologies
 use atincl, sim_year=>syear
@@ -1190,7 +1297,7 @@ enddo
 end subroutine check_chronologies
 
 ! -----------------------------------------------------------------------------
-!
+!> \brief Check that the profiles position is the same over time
 ! -----------------------------------------------------------------------------
 subroutine check_positions
 implicit none
@@ -1247,7 +1354,8 @@ end subroutine check_positions
 
 
 ! -----------------------------------------------------------------------------
-!
+!> \brief Check that the profiles vertical grids heights
+!>      are strictly increasing
 ! -----------------------------------------------------------------------------
 subroutine check_altitudes
 implicit none
@@ -1276,14 +1384,14 @@ enddo
 do i = 1, number_of_files, 1
   do j = 1, sections_per_file, 1
     do k = 2, dynamical_profile_dim
-      if (zt(k-1,j,i).ge.zt(k,j,i)) then
+      if (zd(k-1,j,i).ge.zd(k,j,i)) then
         write(nfecra,*) "the dynamical profile in section ", j
         call bounds(imbrication_files(i),len(imbrication_files(i)),first,last)
         write(nfecra,*) "of the file '",imbrication_files(i)(first:last),"'"
         write(nfecra,*) "is not strictly increasing"
         write(nfecra,*) "erroneous level ",k," with zd =",zd(k,j,i)
         do kk = 1, dynamical_profile_dim, 1
-          write(nfecra,*) "k=",kk,"zd=",zt(kk,j,i)
+          write(nfecra,*) "k=",kk,"zd=",zd(kk,j,i)
         enddo
         stop
       endif
@@ -1294,12 +1402,10 @@ end subroutine check_altitudes
 
 
 ! -----------------------------------------------------------------------------
-!
+!> \brief Compute the hydrostastic pressure by Laplace integration
 ! -----------------------------------------------------------------------------
 subroutine hydrostatic_pressure
-!
-! compute the hydrostastic pressure by Laplace integration
-!
+
 use cstphy, only: tkelvi
 use cstphy, only: rair
 use cstphy, only: gz
@@ -1314,8 +1420,6 @@ integer ih2o
 data ih2o/0/
 save ih2o
 double precision rap
-double precision qsatliq
-external qsatliq
 
 if (ippmod(iatmos).eq.2) ih2o=1
 if(.not.allocated(pressure))then
@@ -1337,9 +1441,9 @@ integration_direction:if (ihpm.eq.0) then
              "hydrostatic_pressure::k=",k
         tmoy = 0.5d0*(tempC(k-1,j,i) + tempC(k,j,i)) + tkelvi
         if (ippmod(iatmos).eq.2) then
-          q0 = min( qw(k-1,j,i), qsatliq( tempC(k-1,j,i)+tkelvi, &
+          q0 = min( qw(k-1,j,i), cs_air_yw_sat( tempC(k-1,j,i), &
                pressure(k-1,j,i)))
-          q1 = min( qw(k  ,j,i), qsatliq( tempC(k  ,j,i)+tkelvi, &
+          q1 = min( qw(k  ,j,i), cs_air_yw_sat( tempC(k  ,j,i), &
                pressure(k-1,j,i)))
         else
           q0 = qw(k-1,j,i)
@@ -1361,9 +1465,9 @@ else
       do k = thermal_profile_dim, 2, -1
         tmoy = 0.5d0*(tempC(k-1,j,i) + tempC(k,j,i)) + tkelvi
         if (ippmod(iatmos).eq.2) then
-          q0 = min( qw(k-1,j,i), qsatliq( tempC(k-1,j,i)+tkelvi, &
+          q0 = min( qw(k-1,j,i), cs_air_yw_sat( tempC(k-1,j,i), &
                pressure(k  ,j,i)))
-          q1 = min( qw(k  ,j,i), qsatliq( tempC(k  ,j,i)+tkelvi, &
+          q1 = min( qw(k  ,j,i), cs_air_yw_sat( tempC(k  ,j,i), &
                pressure(k  ,j,i)))
         else
           q0 = qw(k-1,j,i)
@@ -1396,22 +1500,24 @@ enddo
 end subroutine hydrostatic_pressure
 
 ! -----------------------------------------------------------------------------
-!
+!> \brief Computes the potential_temperature_and_density profiles
 ! -----------------------------------------------------------------------------
 subroutine potential_temperature_and_density
-! computes the potential_temperature_and_density profiles
 use cstphy, only: rair
 use cstphy, only: tkelvi
 use cstphy, only: cp0
+use ctincl, only: cp_a, cp_v
 use atincl
 implicit none
 double precision rhum! I prefer bourbon
 integer ih2o
 data ih2o/0/
 save ih2o
-double precision rscp
+double precision rscp, cpvcpa
 integer i,j,k
 
+cpvcpa = cp_v / cp_a
+
 if (.not.allocated(theta)) then
   allocate(theta(thermal_profile_dim,sections_per_file,number_of_files))
 endif
@@ -1458,17 +1564,30 @@ do i = 1, number_of_files, 1
  end subroutine potential_temperature_and_density
 
 ! -----------------------------------------------------------------------------
-!
-! -----------------------------------------------------------------------------
+!> \brief Search for the position of a value in an array,
+!> assuming that the array is sorted in a strictly increasing order
+!>
+!> return if possible lower,upper such that :
+!>-                    the_array(lower) <= the_value <= the_array(upper)
+!>        otherwise :
+!>          lower==upper if the_value<the_array(first)
+!>                       or if the_value>the_array(last)
+!>          lower> upper if none of the previous cases applies (anomaly)
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]       the_array   an array
+!> \param[in]       the_value   a particular value
+!> \param[in]       lower       the index of the first membre of the array
+!>                                  lower than the value
+!> \param[in]       upper       the index of the first membre of the array
+!>                                  greater than the value
+!-------------------------------------------------------------------------------
 subroutine get_index(the_array,the_value,lower,upper)
-! assuming the_array a sorted, strictly increasing array of values
-! return if possible lower,upper such that :
-!                    the_array(lower)<=the_value<=the_array(upper)
-!        otherwise :
-!          lower==upper if the_value<the_array(first)
-!                       or if the_value>the_array(last)
-!          lower> upper if none of the previous cases applies (anomaly)
 implicit none
+
 double precision,dimension(:),intent(in) :: the_array
 double precision,intent(in) :: the_value
 integer,intent(out) ::upper,lower
@@ -1500,14 +1619,22 @@ lower = dmax
 upper = dmin
 end subroutine get_index
 
-! *****************************************************************************
-! *
-! *****************************************************************************
+! -----------------------------------------------------------------------------
+!> \brief  Interpolates a "profile" at a given time.
+!> Given a series of profiles varying in time
+!> you get the profile interpolated from them at the given time.
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   the_time                current time
+!> \param[in]   the_times               times array
+!> \param[in]   the_profiles            input profiles
+!> \param[out]   interpolated_profile    output profile
+!-------------------------------------------------------------------------------
 subroutine time_interpolation(the_time,the_times, &
      the_profiles,interpolated_profile)
-! interpolates a "profile" at a given time.
-! given a series of profiles varying in time
-! you get the profile interpolated from them at the given time.
 implicit none
 double precision,intent(in) :: the_time
 double precision, dimension(:),intent(in) :: the_times
@@ -1541,12 +1668,20 @@ endif
 end subroutine time_interpolation
 
 
-! *****************************************************************************
-! *
-! *****************************************************************************
+!-------------------------------------------------------------------------------
+!> \brief interpolates in a profile at a given altitude
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   the_altitude        current height
+!> \param[in]   the_altitudes       height array
+!> \param[in]   the_profile         the profile
+!> \param[out]   interpolated_value  interpolated profile
+!-------------------------------------------------------------------------------
 subroutine altitude_interpolation(the_altitude,the_altitudes,the_profile, &
      interpolated_value)
-! interpolates in a profile at a given altitude
 implicit none
 double precision,intent(in) :: the_altitude
 double precision, dimension(:),intent(in) :: the_altitudes
@@ -1583,7 +1718,7 @@ end subroutine altitude_interpolation
 
 
 ! -----------------------------------------------------------------------------
-!
+!> \brief  Compute radius of influence
 ! -----------------------------------------------------------------------------
 subroutine red_tape
 implicit none
@@ -1609,13 +1744,19 @@ enddo
 end subroutine red_tape
 
 ! -----------------------------------------------------------------------------
-!
+!> \brief  Identification of the first and last non white character
+!>      of a string
 ! -----------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   string      the input string
+!> \param[in]   length      its length
+!> \param[out]  b           number of the first non white character
+!> \param[out]  e           number of the last non white character
+!-------------------------------------------------------------------------------
 subroutine bounds(string,length,b,e)
-! very low level for printing strings
-! length: input=the length of the input string
-! b=output=first non white from beginning of the string
-! e=output=first non white from end of the string
 !
 ! Being compelled to write such low level stuff because
 ! the designers of the language didn't bother to offer
@@ -1648,19 +1789,22 @@ return
 end subroutine bounds
 
 
-! ****************************************************************
-!
-! ****************************************************************
-subroutine activate_imbrication
-! ----------------------------------------------------------------
-!
-! ----------------------------------------------------------------
-implicit none
-integer i,j,k
-integer first,last
-! ----------------------------------------------------------------
-!
-! ----------------------------------------------------------------
+! -----------------------------------------------------------------------------
+!> \brief Prepare data for imbrication by reading meteo files
+!>
+!> Warning : the list of files is supposed to be "imbrication_files_list.txt"
+!>
+! -----------------------------------------------------------------------------
+subroutine activate_imbrication() &
+  bind(C, name='cs_f_activate_imbrication')
+
+  use cs_c_bindings
+
+  implicit none
+
+  integer i,j,k
+  integer first,last
+
 write(nfecra,*)"*******************************"
 write(nfecra,*)"Atmospheric Imbrication:       "
 write(nfecra,*)"*******************************"
@@ -1742,40 +1886,57 @@ end subroutine activate_imbrication
 
 
 ! --------------------------------------------------------------
-!
-! --------------------------------------------------------------
-subroutine summon_cressman(the_time)
-implicit none
-double precision the_time
-logical first_call
-data first_call /.true./
-save first_call
-integer lb1, ub1
-integer lb2, ub2
-integer nbmes
-integer i,j
-integer, dimension(:,:), allocatable :: ones
-if (first_call) then
-  if (cressman_u) then
-    call mestcr("u",len("u"),1,0,id_u)
+!> \brief Prepare for the cressman interpolation of the variables
+! -----------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   the_time        current time
+!-------------------------------------------------------------------------------
+
+subroutine summon_cressman(the_time) &
+  bind(C, name='cs_summon_cressman')
+  use, intrinsic :: iso_c_binding
+  implicit none
+
+  procedure() :: mestcr, mesmap
+
+  ! Arguments
+  real(c_double), value :: the_time
+
+  ! Local variables
+
+  logical first_call
+  data first_call /.true./
+  save first_call
+  integer lb1, ub1
+  integer lb2, ub2
+  integer nbmes
+  integer i,j
+  integer, dimension(:,:), allocatable :: ones
+
+  if (first_call) then
+    if (cressman_u) then
+    call mestcr("u"//c_null_char, 1,0, id_u)
   endif
   if (cressman_v) then
-    call mestcr("v",len("v"),1,0,id_v)
+    call mestcr("v"//c_null_char, 1,0, id_v)
   endif
   if (cressman_tke) then
-    call mestcr("tke",len("tke"),1,0,id_tke)
+    call mestcr("tke"//c_null_char, 1,0, id_tke)
   endif
   if (cressman_eps) then
-    call mestcr("eps",len("eps"),1,0,id_eps)
+    call mestcr("eps"//c_null_char, 1, 0, id_eps)
   endif
   if (cressman_theta .and. ippmod(iatmos).ge.1) then
-    call mestcr("theta",len("theta"),1,0,id_theta)
+    call mestcr("theta"//c_null_char, 1, 0, id_theta)
   endif
   if (cressman_qw .and. ippmod(iatmos).ge.2) then
-    call mestcr("qw",len("qw"),1,0,id_qw)
+    call mestcr("qw"//c_null_char, 1, 0, id_qw)
   endif
   if (cressman_nc.and.ippmod(iatmos).ge.2) then
-    call mestcr("nc",len("nc"),1,0,id_nc)
+    call mestcr("nc"//c_null_char, 1, 0, id_nc)
   endif
   call red_tape
   first_call = .false.
@@ -2014,7 +2175,7 @@ if (cressman_theta.and.ippmod(iatmos).ge.1) then
   deallocate(ones)
 endif
 
-if (cressman_qw.and.ippmod(iatmos).ge.1)then
+if (cressman_qw.and.ippmod(iatmos).ge.2)then
   lb1 = lbound(ti_qw,1)
   lb2 = lbound(ti_qw,2)
   ub1 = ubound(ti_qw,1)
