@@ -1,8 +1,8 @@
 !-------------------------------------------------------------------------------
 
-! This file is part of Code_Saturne, a general-purpose CFD tool.
+! This file is part of code_saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2016 EDF S.A.
+! Copyright (C) 1998-2024 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -19,33 +19,19 @@
 ! Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 !-------------------------------------------------------------------------------
-
-subroutine atlecm &
-     !================
-
-     ( imode )
-
-!===============================================================================
-!  Purpose:
-!  -------
-!             Reads the meteo profile data
-!             for the atmospheric module
-!
-!             IMODE = 0 : READING FOR DIMENSIONS ONLY
-!             IMODE = 1 : READING ACTUAL METEO DATA
-!
-!             WARNING : METEO DATA STRUCTURE CAN BE FOUND IN MODULE ATINCL
+!> \file atlecm.f90
+!> \brief Reads the meteo profile data for the atmospheric module
 !-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-!__________________!____!_____!________________________________________________!
-
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
-!===============================================================================
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   imode        0: reading for dimensions and starting time only
+!>                           1: reading actual meteo data
+!-------------------------------------------------------------------------------
+subroutine atlecm (imode) &
+  bind(C, name="cs_f_read_meteo_profile")
+  use, intrinsic :: iso_c_binding
 
 !===============================================================================
 ! Module files
@@ -57,22 +43,29 @@ use entsor
 use cstnum
 use cstphy
 use ppppar
+use optcal
 use atincl
+use ctincl, only: cp_a, cp_v
+use cs_c_bindings
 
 !===============================================================================
 
 implicit none
 
+procedure() :: comp_quantile
+
 ! Arguments
 
-integer           imode
+integer(c_int), value :: imode
 
 ! a function used in the routine
 ! for the diagnostic of liquid water content
-double precision qsatliq
 
 ! Local variables
 
+character(len=80) :: ficmet
+
+integer impmet
 integer itp, ii, ios, k
 integer year, quant,hour,minute, month, day
 integer ih2o
@@ -82,6 +75,9 @@ double precision sjday, jday
 double precision rap,rscp,tmoy, rhmoy
 double precision ztop, zzmax, tlkelv, pptop, dum
 double precision rhum,q0,q1
+double precision cpvcpa
+double precision pres, temp, qv
+double precision p_ref, t_ref
 
 character(len=80) :: ccomnt,oneline
 character(len=1)  :: csaute
@@ -98,24 +94,22 @@ endif
 ! 0. Initialization
 !===============================================================================
 
-CSAUTE = '/'
+csaute = '/'
 
 ! --> Opens the meteo file
-open (unit=impmet, file=ficmet,                                  &
+call atmo_get_meteo_file_name(ficmet)
+
+open (newunit=impmet, file=ficmet,                                  &
      status='old', form='formatted', access='sequential',       &
      iostat=ios, err=99)
 rewind(unit=impmet, err=99)
 
 itp=0
 ih2o = 0
+!--> flag to take into account the humidity
+if (ippmod(iatmos).eq.2) ih2o=1
 
-if (imode.eq.1) then
-  rscp=rair/cp0
-
-  !--> flag to take into account the humidity
-  if (ippmod(iatmos).eq.2) ih2o=1
-
-endif
+cpvcpa = cp_v / cp_a
 
 !===============================================================================
 ! 1. Loop on time
@@ -141,41 +135,39 @@ backspace(impmet)
 ! --> year, quant-day, hour, minute, second  of the profile (UTC)
 ! NB: second is real, all others are integers
 
-if (imode.eq.0) then
-  read(impmet, *, err=999, end=906)
-else
-  second = -9999.d0
-  read(impmet, '(a80)', err=999, end=906) oneline
-  read(oneline, *, err=907, end=907) year, month, day,  &
-                                                  hour, minute, second
+second = -9999.d0
+read(impmet, '(a80)', err=999, end=906) oneline
+read(oneline, *, err=907, end=907) year, month, day,  &
+                                                hour, minute, second
 ! --> catch some read errors
-  if (month.gt.12.or.day.gt.31) then
-    write(nfecra,8005)
-    call csexit (1)
-  endif
-  call comp_quantile(day, month, year, quant)
-  goto 908
+if (month.gt.12.or.day.gt.31) then
+  write(nfecra,8005)
+  call csexit (1)
+endif
+call comp_quantile(day, month, year, quant)
+goto 908
 907  continue
-  read(oneline, *, err=999, end=906) year, quant, hour, minute, second
+read(oneline, *, err=999, end=906) year, quant, hour, minute, second
 908 continue
 ! --> catch some read errors
-  if (second.lt.0d0.or.quant.gt.366) then
-    write(nfecra,8005)
-    call csexit (1)
-  endif
+if (second.lt.0d0.or.quant.gt.366) then
+  write(nfecra,8005)
+  call csexit (1)
+endif
 
-  ! --> if the date and time are not completed in usati1.f90,
-  !     the date and time of the first meteo profile are taken as the
-  !     starting time of the simulation
+! --> if the date and time are not completed in cs_user_model
+!     the date and time of the first meteo profile are taken as the
+!     starting time of the simulation
 
-  if (syear.lt.0) then
-    syear = year
-    squant = quant
-    shour = hour
-    smin = minute
-    ssec = second
-  endif
+if (syear.lt.0) then
+  syear = year
+  squant = quant
+  shour = hour
+  smin = minute
+  ssec = second
+endif
 
+if (imode.ne.0) then
   !--> Compute the relative time to the starting time of the simulation
 
   ! --> Compute the julian day for the starting day of the simulation
@@ -204,7 +196,6 @@ else
       !==========
     endif
   endif
-
 endif
 
 !===============================================================================
@@ -220,7 +211,7 @@ backspace(impmet)
 if (imode.eq.0) then
   read(impmet, *, err=999, end=999)
 else
-  read(impmet, *, err=999, end=999) xmet(itp), ymet(itp)
+  read(impmet, *, err=999, end=999) xyp_met(1, itp), xyp_met(2, itp)
 endif
 
 !===============================================================================
@@ -232,10 +223,15 @@ endif
 if (ccomnt(1:1).eq.csaute) go to 103
 backspace(impmet)
 
-if (imode.eq.0) then
-  read(impmet, *, err=999, end=999)
-else
-  read(impmet, *, err=999, end=999) pmer(itp)
+read(impmet, *, err=999, end=999) pres
+
+! p_sea = p0
+if (itp.eq.1) then
+  p0 = pres
+endif
+
+if (imode.eq.1) then
+  xyp_met(3, itp) = pres
 endif
 
 !===============================================================================
@@ -253,15 +249,21 @@ if (imode.eq.0) then
   if (nbmett.le.1) then
     write(nfecra, 8001)
     call csexit (1)
-    !==========
   endif
 
   do ii = 1, nbmett
-    read (impmet,*,err=999,end=999) zzmax
+    read (impmet,*,err=999,end=999) zzmax, temp, qv
+
+    ! Initialize p0, rho0 and theta0 at the first level
+    if (ii.eq.1 .and. itp.eq.1) then
+      t0 = temp + tkelvi
+      rhum = rair*(1.d0+(rvsra-1.d0)*qv*ih2o)
+      ro0 = p0 / t0 /rhum
+    endif
   enddo
 
   !-->  Computes nbmaxt:
-  !     if 1d radiative model (iatra1 = 1) altitudes are completed up to 11000m
+  !     if 1D radiative model (iatra1 = 1) altitudes are completed up to 11000m
   !     (i.e. nbmaxt > nbmett, used for dimensions of ttmet and qvmet array)
   !     if no radiative model nbmaxt = nbmett
   nbmaxt = nbmett
@@ -291,6 +293,14 @@ else
            ttmet(ii,itp),qvmet(ii,itp)
     endif
 
+    ! Initialize p0, rho0 and theta0 at the first level
+    if (ii.eq.1 .and. itp.eq.1) then
+      qv = qvmet(ii,itp)
+      t0 = ttmet(ii,itp) + tkelvi
+      rhum = rair*(1.d0+(rvsra-1.d0)*qv*ih2o)
+      ro0 = p0 / t0 /rhum
+    endif
+
     !--> Check the unity of the specific humidity (kg/kg) when used
 
     if (qvmet(ii,itp).gt.0.1 .or. qvmet(ii,itp).lt.0.) then
@@ -313,12 +323,20 @@ else
     ztop = 11000.d0
     ii = nbmett
     zzmax = (int(ztmet(ii))/1000)*1000.d0
-    ii = nbmett
+    p_ref = 101325.d0
+    t_ref = 288.15d0
     do while(zzmax.le.(ztop-1000.d0))
       zzmax = zzmax + 1000.d0
       ii = ii + 1
       ztmet(ii) = zzmax
-      call atmstd(ztmet(ii), dum, tlkelv, dum) ! standard temperature profile
+      ! standard temperature profile above the domaine
+      if (ihpm.eq.0) then
+        call atmstd(ztmet(nbmett), p0, ttmet(nbmett,itp)+tkelvi, &
+                    ztmet(ii), dum, tlkelv, dum)
+      else ! if (ihpm.eq.1) then
+       call atmstd(0.d0, p_ref, t_ref, &
+                   ztmet(ii), dum, tlkelv, dum)
+      endif
       ttmet(ii,itp) = tlkelv - tkelvi
       if (iqv0.eq.0) then
         qvmet(ii,itp) = 0.d0   ! standard atmosphere: q=0
@@ -339,24 +357,27 @@ endif
 !===============================================================================
 ! If ihpm = 0 (default): bottom to top Laplace integration based on pressure at
 ! sea level (pmer(itp))
-! If ihpm = 1 (usati1): top to bottom Laplace integration based on pressure at
+! If ihpm = 1 (hydrostatic_pressure_model in cs_user_parameters):
+! top to bottom Laplace integration based on pressure at
 ! the top of the domain (ztmet(nbmaxt)) for the standard atmosphere
 
 if (imode.eq.1) then
 
-  phmet(1, itp) = pmer(itp)
+  phmet(1, itp) = xyp_met(3, itp)
   rscp = rair/cp0
+  p_ref = 101325.d0
+  t_ref = 288.15d0
 
   if (ihpm.eq.0) then
-    phmet(1,itp) = pmer(itp)
+    phmet(1,itp) = xyp_met(3, itp)
     do k = 2, nbmaxt
       tmoy = 0.5d0*(ttmet(k-1,itp) + ttmet(k,itp)) + tkelvi
 
-      if(ippmod(iatmos).eq.2) then ! take liquid water into account
-        q0 = min( qvmet(k-1,itp), qsatliq( ttmet(k-1,itp) &
-            + tkelvi, phmet(k-1,itp)))
-        q1 = min( qvmet(k  ,itp), qsatliq( ttmet(k  ,itp) &
-            + tkelvi, phmet(k-1,itp)))
+      if (ippmod(iatmos).eq.2) then ! take liquid water into account
+        q0 = min( qvmet(k-1,itp), cs_air_yw_sat( ttmet(k-1,itp) &
+            , phmet(k-1,itp)))
+        q1 = min( qvmet(k  ,itp), cs_air_yw_sat( ttmet(k  ,itp) &
+            , phmet(k-1,itp)))
         !in q1=.. phmet(k-1,itp) is not a mistake: we can not use phmet(k,itp)
         !since this is what we want to estimate.
       else
@@ -370,14 +391,16 @@ if (imode.eq.1) then
       phmet(k,itp) = phmet(k-1,itp)*exp(rap)
     enddo
   else
-    call atmstd (ztmet(nbmaxt), pptop, dum, dum) ! standard profile temperature
+    ! Standard pressure profile above the domain
+    call atmstd (0.d0, p_ref, t_ref, &
+                 ztmet(nbmaxt), pptop, dum, dum)
     phmet(nbmaxt,itp) = pptop
     do k = nbmaxt-1, 1, -1
       tmoy = 0.5d0*(ttmet(k+1,itp) + ttmet(k,itp)) + tkelvi
 
       if(ippmod(iatmos).eq.2) then ! take liquid water into account
-        q0 = min( qvmet(k,itp), qsatliq( ttmet(k,itp)+tkelvi, phmet(k+1,itp)))
-        q1 = min( qvmet(k+1  ,itp), qsatliq( ttmet(k+1,itp)+tkelvi, phmet(k+1,itp)))
+        q0 = min( qvmet(k,itp), cs_air_yw_sat( ttmet(k,itp), phmet(k+1,itp)))
+        q1 = min( qvmet(k+1  ,itp), cs_air_yw_sat( ttmet(k+1,itp), phmet(k+1,itp)))
         !in q0=.. phmet(k+1,itp) is not a mistake: we can not use phmet(k,itp)
         !since this is what we want to estimate.
       else
@@ -457,7 +480,7 @@ endif
 ! 9. Printings
 !===============================================================================
 
-if (imode.eq.1) then
+if (imode.eq.1.and.imeteo.eq.1) then
   if (itp.eq.1) then
     write(nfecra, *)
     write(nfecra, *) '==================================================='
@@ -468,7 +491,7 @@ if (imode.eq.1) then
 7995 format(1x, i4, i5, 2i4, f10.2)
   write(nfecra, *) 'tmmet(itp):'
   write(nfecra, 7996) tmmet(itp)
-7996 format(1x, f10.2)
+7996 format(1x, e10.3)
   write(nfecra, *) 'zdmet, umet, vmet, ekmet, epmet:'
   do ii = 1, nbmetd
     write(nfecra, 7997)                                            &
@@ -495,7 +518,7 @@ if (imode.eq.1) then
       write(nfecra,7999)                                            &
            ztmet(ii), ttmet(ii,itp), tpmet(ii,itp),              &
            rmet(ii,itp), phmet(ii,itp), qvmet(ii,itp),           &
-           qsatliq(ttmet(ii,itp)+tkelvi , phmet(ii,itp)),         &
+           cs_air_yw_sat(ttmet(ii,itp) , phmet(ii,itp)),         &
            ncmet(ii,itp)
 7999  format(1x, 3f8.2,f8.4,f12.3,e10.3,e10.3,e12.5)
     enddo
@@ -539,147 +562,6 @@ call csexit (1)
 ! Formats
 !--------
 
-#if defined(_CS_LANG_FR)
-8000 format (                                                    &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecm)      ',/,&
-     '@    =========                                               ',/,&
-     '@      PHYSIQUE PARTICULIERE ATMOSPHERIQUE                   ',/,&
-     '@                                                            ',/,&
-     '@              Erreur  dans le fichier meteo:                ',/,&
-     '@      ordre chronologique des profils non respecte          ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/)
-8001 format (                                                    &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecm)      ',/,&
-     '@    =========                                               ',/,&
-     '@      PHYSIQUE PARTICULIERE ATMOSPHERIQUE                   ',/,&
-     '@                                                            ',/,&
-     '@              Erreur  dans le fichier meteo:                ',/,&
-     '@  le nombre de mesures de temperatures doit etre            ',/,&
-     '@  superieur a 2                                             ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/)
-8002 format (                                                    &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecm)      ',/,&
-     '@    =========                                               ',/,&
-     '@      PHYSIQUE PARTICULIERE ATMOSPHERIQUE                   ',/,&
-     '@                                                            ',/,&
-     '@              Erreur  dans le fichier meteo:                ',/,&
-     '@  le nombre de mesures de vitesses doit etre                ',/,&
-     '@  superieur a 2                                             ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/)
-8003 format (                                                    &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ATTENTION:VERIFICIATION A L''ENTREE DES DONNEES (atlecm)',/,&
-     '@   =========                                               ',/,&
-     '@        PHYSIQUE PARTICULIERE ATMOSPHERIQUE                 ',/,&
-     '@                                                            ',/,&
-     '@     erreur a la lecture du fichier meteo                   ',/,&
-     '@  humidites specifiques irrealistes                         ',/,&
-     '@  verifier les unites (kg/kg)                               ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                             ',/)
-8004 format (                                                    &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ATTENTION:VERIFICATION A L''ENTREE DES DONNEES (atlecm)' ,/,&
-     '@   =========                                               ',/,&
-     '@        PHYSIQUE PARTICULIERE ATMOSPHERIQUE                 ',/,&
-     '@                                                            ',/,&
-     '@  erreur a la lecture du fichier meteo                      ',/,&
-     '@  nombre de gouttes lus <0                                  ',/,&
-     '@  verifier le fichier meteo                                 ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                             ',/)
-8005 format (                                                    &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ATTENTION:VERIFICATION A L''ENTREE DES DONNEES (atlecm)  ',/,&
-     '@   =========                                                ',/,&
-     '@        PHYSIQUE PARTICULIERE ATMOSPHERIQUE                 ',/,&
-     '@                                                            ',/,&
-     '@  erreur a la lecture de la date du fichier meteo           ',/,&
-     '@  verifier le format (entiers, reel)                        ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                             ',/)
-9998 format(                                                     &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecm)      ',/,&
-     '@    =========                                               ',/,&
-     '@      PHYSIQUE PARTICULIERE ATMOSPHERIQUE                   ',/,&
-     '@                                                            ',/,&
-     '@  Erreur a l''ouverture du fichier meteo                    ',/,&
-     '@  Verifier le nom du fichier meteo                          ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/)
-9999 format(                                                     &
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/,&
-     '@ @@ ATTENTION : ARRET A L''ENTREE DES DONNEES (atlecm)      ',/,&
-     '@    =========                                               ',/,&
-     '@      PHYSIQUE PARTICULIERE ATMOSPHERIQUE                   ',/,&
-     '@                                                            ',/,&
-     '@  Erreur a la lecture du fichier meteo.                     ',/,&
-     '@    Le fichier a ete ouvert mais est peut etre incomplet    ',/,&
-     '@    ou son format inadapte.                                 ',/,&
-     '@                                                            ',/,&
-     '@    year (integer), quantile (integer), hour (integer),    ',/,&
-     '@          minute (integer), second (dble prec) of the profile',/,&
-     '@    location of the meteo profile (x,y) (dble prec)         ',/,&
-     '@    sea level pressure (double precision)                   ',/,&
-     '@ temperature profile:                                       ',/,&
-     '@   number of altitudes (integer)                            ',/,&
-     '@   alt.,temperature  in celcius,humidity in kg/kg (dble prec)',/,&
-     '@ wind profile:                                              ',/,&
-     '@   number of altitudes (integer)                            ',/,&
-     '@   alt.,u,v,k,eps (double precision)                        ',/,&
-     '@ NO LINE AT THE END OF THE FILE                             ',/,&
-     '@                                                            ',/,&
-     '@  Le calcul ne sera pas execute.                            ',/,&
-     '@                                                            ',/,&
-     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-     '@                                                            ',/)
-
-#else
-
 8000 format (                                                    &
      '@                                                            ',/,&
      '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -817,36 +699,22 @@ call csexit (1)
      '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
      '@                                                            ',/)
 
-#endif
-
 end subroutine atlecm
 
 
 !===============================================================================
-
-subroutine comp_quantile(jour,mois,annee,quant)
-
-!===============================================================================
-!  Purpose:
-!  -------
-!             Computes quantile
+!> \brief Compute the calendar day number from the date
 !-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-!__________________!____!_____!________________________________________________!
-
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
-!===============================================================================
-
-!===============================================================================
-! Module files
-!===============================================================================
-
-!===============================================================================
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]       jour        day
+!> \param[in]       mois        month
+!> \param[in]       annee       year
+!> \param[out]      quant       calendar day number
+!-------------------------------------------------------------------------------
+subroutine comp_quantile(jour,mois,annee,quant)
 
 implicit none
 
@@ -857,7 +725,6 @@ integer quant
 ! Local variables
 integer distrib, booll, retrait
 
-
 distrib = int(mois * 275 / 9.) - 30
 booll = int((mois + 9 ) / 12.) !boolean=0 for jan and feb and =1 for march to dec
 retrait = 1+int(mod(annee,4)+2)/3 !retrait=2 days non leap years and =1 day for leap years
@@ -865,5 +732,3 @@ quant = distrib-(booll*retrait)+jour
 
 return
 end subroutine comp_quantile
-
-
