@@ -6,9 +6,9 @@
  *============================================================================*/
 
 /*
-  This file is part of Code_Saturne, a general-purpose CFD tool.
+  This file is part of code_saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2025 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,7 +32,11 @@
  *============================================================================*/
 
 #if defined(HAVE_CONFIG_H)
-#  include "cs_config.h"
+#  include "base/cs_config.h"
+#endif
+
+#ifdef __cplusplus
+#include <type_traits>
 #endif
 
 /*============================================================================
@@ -58,6 +62,11 @@ extern "C" {
 
 #else
 
+#  define _LIBINTL_H /* Prevent inclusion of <libintl.h> by other files
+                        with incorrect or missing checks;
+                        TODO locate files causing issues to avoid
+                        requiring this workaround */
+
 #  define _(String) (String)
 #  define N_(String) String
 #  define textdomain(String) (String)
@@ -92,7 +101,24 @@ extern "C" {
 #endif
 
 #if defined(HAVE_OPENMP)
+
 #  include <omp.h>
+
+#if _OPENMP >= 201307     /* simd construct available from OpenMP 4.0 */
+#undef HAVE_OPENMP_SIMD
+#define HAVE_OPENMP_SIMD 1
+#endif
+
+#endif
+
+/* Do we have accelerator support ? */
+
+#if defined(HAVE_CUDA)
+#define HAVE_ACCEL 1
+#elif defined(HAVE_SYCL)
+#define HAVE_ACCEL 1
+#elif defined(HAVE_OPENMP_TARGET)
+#define HAVE_ACCEL 1
 #endif
 
 /*============================================================================
@@ -126,12 +152,28 @@ extern "C" {
 
 #else /* C++ */
 
-#  ifndef HAVE_RESTRICT /* Must be provided by caller */
-#    define restrict
+#  if defined(__GNUC__) || defined(__clang__) || defined (__NVCC__)
+#    define restrict __restrict__
+#  elif defined(_MSC_VER)
+#    define restrict __restrict
+#  else
+#    ifndef HAVE_RESTRICT /* Must be provided by caller */
+#      define restrict
+#    endif
 #  endif
 
 #endif /* __cplusplus */
 
+/* Definition of a DEPRECATED decorator which may be called in the code */
+
+#if defined(__GNUC__) || defined(__clang__)
+#  define DEPRECATED __attribute__((deprecated))
+#elif defined(_MSC_VER)
+#  define DEPRECATED __declspec(deprecated)
+#else
+#  define DEPRECATED
+#endif
+
 /*============================================================================
  * Definitions that may not always be provided directly by the system
  *============================================================================*/
@@ -185,6 +227,12 @@ extern "C" {
 #  define __bool_true_false_are_defined 1
 #endif
 
+/* C++ assert necessary for template */
+#if defined(__cplusplus)
+#include <typeinfo>
+#include "assert.h"
+#endif
+
 /* int32_t type */
 
 #if !defined(HAVE_INT32_T)
@@ -238,7 +286,7 @@ typedef unsigned long long uint64_t;
 #endif
 
 /*============================================================================
- * General types and macros used throughout Code_Saturne
+ * General types and macros used throughout code_saturne
  *============================================================================*/
 
 #ifdef __cplusplus
@@ -258,6 +306,7 @@ typedef enum {
   CS_CHAR,               /* character values */
   CS_FLOAT,              /* 4-byte floating point values */
   CS_DOUBLE,             /* 8-byte floating point values */
+  CS_UINT16,             /* 2-byte unsigned integer values */
   CS_INT32,              /* 4-byte signed integer values */
   CS_INT64,              /* 8-byte signed integer values */
   CS_UINT32,             /* 4-byte unsigned integer values */
@@ -266,7 +315,7 @@ typedef enum {
 } cs_datatype_t;
 
 /*----------------------------------------------------------------------------
- * Basic types used by Code_Saturne
+ * Basic types used by code_saturne
  * They may be modified here to better map to a given library, with the
  * following constraints:
  *  - cs_lnum_t must be signed
@@ -276,52 +325,93 @@ typedef enum {
 /* Global integer index or number */
 
 #if defined(HAVE_LONG_GNUM)
-  #if (SIZEOF_LONG == 8)
-    typedef unsigned long       cs_gnum_t;
-  #elif (SIZEOF_LONG_LONG == 8)
-    typedef unsigned long long  cs_gnum_t;
-  #else
-    #error
-  #endif
+  typedef uint64_t  cs_gnum_t;
 #else
   typedef unsigned  cs_gnum_t;
 #endif
 
+/* Local integer index or number */
+
+#if defined(HAVE_LONG_LNUM)
+  typedef int64_t  cs_lnum_t;
+#else
+  typedef int  cs_lnum_t;
+#endif
+
 /* Other types */
 
-typedef int      cs_lnum_t;     /* Local integer index or number */
-typedef double   cs_coord_t;    /* Real number (coordinate value) */
+typedef double             cs_coord_t;   /* Real number (coordinate value) */
 
-typedef int      cs_int_t;      /* Fortran integer */
-typedef double   cs_real_t;     /* Fortran double precision */
-typedef char     cs_byte_t;     /* Byte (untyped memory unit) */
+typedef double              cs_real_t;   /* Fortran double precision */
+typedef char                cs_byte_t;   /* Byte (untyped memory unit) */
+typedef unsigned short int  cs_flag_t;   /* Flag storing metadata */
+
+typedef double              cs_nreal_t;  /* Real number (normalized value) */
+typedef double              cs_dreal_t;  /* Local distance */
+typedef double              cs_rreal_t;  /* Reconstruction distance */
 
 /* Vector or array block types */
 
-typedef int        cs_lnum_2_t[2];      /* Vector of 2 local numbers */
+typedef cs_lnum_t  cs_lnum_2_t[2];      /* Vector of 2 local numbers */
+typedef cs_lnum_t  cs_lnum_3_t[3];      /* Vector of 3 local numbers */
 
-typedef double     cs_coord_3_t[3];     /* Vector of 3 real (coordinate)
-                                           values */
+typedef cs_coord_t cs_coord_3_t[3];         /* Vector of 3 real (coordinate)
+                                               values */
 
-typedef cs_real_t  cs_real_2_t[2];      /* Vector of 2 real values */
-typedef cs_real_t  cs_real_3_t[3];      /* Vector of 3 real values */
-typedef cs_real_t  cs_real_4_t[4];      /* Vector of 4 real values */
-typedef cs_real_t  cs_real_6_t[6];      /* Vector of 6 real values
-                                           (for symmetric tensor) */
-typedef cs_real_t  cs_real_33_t[3][3];  /* Matrix of 3x3 real values */
-typedef cs_real_t  cs_real_66_t[6][6];  /* Matrix of 6x6 real values */
+typedef cs_real_t  cs_real_2_t[2];          /* Vector of 2 real values */
+typedef cs_real_t  cs_real_3_t[3];          /* Vector of 3 real values */
+typedef cs_real_t  cs_real_4_t[4];          /* Vector of 4 real values */
+typedef cs_real_t  cs_real_6_t[6];          /* Vector of 6 real values
+                                               (for symmetric tensor) */
+typedef cs_real_t  cs_real_9_t[9];          /* Vector of 9 real values */
+typedef cs_real_t  cs_real_10_t[10];        /* Vector of 10 real values */
 
-typedef cs_real_t  cs_real_34_t[3][4];  /* Matrix of 3x4 real values */
+typedef cs_real_t  cs_real_23_t[2][3];      /* Matrix of 2x3 real values */
 
-typedef cs_real_33_t  cs_real_332_t[2];  /* vector of 2 3x3 matrices
-                                            of real values */
+typedef cs_real_t  cs_real_33_t[3][3];      /* Matrix of 3x3 real values */
+typedef cs_real_t  cs_real_66_t[6][6];      /* Matrix of 6x6 real values */
+typedef cs_real_t  cs_real_99_t[9][9];      /* Matrix of 9x9 real values */
+
+typedef cs_real_t  cs_real_333_t[3][3][3];  /* tensor of 3x3x3 real values */
+
+typedef cs_real_t  cs_real_34_t[3][4];      /* Matrix of 3x4 real values */
+
+typedef cs_real_t  cs_real_63_t[6][3];      /* Matrix of 6x3 real values */
+
+typedef cs_real_t  cs_real_69_t[6][9];      /* Matrix of 6x9 real values */
+
+typedef cs_real_33_t  cs_real_332_t[2];     /* vector of 2 3x3 matrices
+                                               of real values */
+typedef cs_real_66_t  cs_real_662_t[2];     /* vector of 2 6x6 matrices
+                                               of real values */
+
+typedef cs_nreal_t  cs_nreal_3_t[3];        /* Vector of normalized real values
+                                               (i.e. unit vector) */
+typedef cs_dreal_t  cs_dreal_3_t[3];        /* Vector joining 2 local coordinates */
+typedef cs_rreal_t  cs_rreal_3_t[3];        /* Reconstruction distance Vector */
+
+typedef struct {
+
+  double   val;  /* Value */
+  int      id;   /* Id related to value */
+
+} cs_double_int_t;
+
+/* Vector-valued quantity stored using its measure (i.e. length) and
+   its direction given by a unitary vector */
+
+typedef struct {
+
+  double  meas;
+  double  unitv[3];
+
+} cs_nvec3_t;
 
 /* Mappings to MPI datatypes */
 /*---------------------------*/
 
 #if defined(HAVE_MPI) && !defined(CS_IGNORE_MPI)
 
-#  define CS_MPI_INT       MPI_INT         /* If cs_int_t is an int */
 #  define CS_MPI_REAL      MPI_DOUBLE      /* If cs_real_t is a double */
 
 /* MPI type for cs_gnum_t integer type (depends on configuration) */
@@ -343,12 +433,21 @@ typedef cs_real_33_t  cs_real_332_t[2];  /* vector of 2 3x3 matrices
 #    define CS_MPI_GNUM       MPI_UNSIGNED
 #  endif
 
-#  define CS_MPI_LNUM     MPI_INT         /* MPI type for cs_lnum_t type */
-#  define CS_MPI_COORD    MPI_DOUBLE      /* MPI type for cs_coord_t type */
+/* MPI type for cs_lnum_t type */
+
+#  if defined(HAVE_LONG_LNUM)
+#    define CS_MPI_LNUM     MPI_LONG
+#  else
+#    define CS_MPI_LNUM     MPI_INT
+#  endif
+
+#  define CS_MPI_EFLAG    MPI_UNSIGNED       /* MPI type for cs_mflag_t type */
+#  define CS_MPI_FLAG     MPI_UNSIGNED_SHORT /* MPI type for cs_flag_t type */
+#  define CS_MPI_COORD    MPI_DOUBLE         /* MPI type for cs_coord_t type */
 
 #endif /* defined(HAVE_MPI) && !defined(CS_IGNORE_MPI) */
 
-/* Mappings to Code_Saturne datatypes */
+/* Mappings to code_saturne datatypes */
 /*------------------------------------*/
 
 #if defined(HAVE_LONG_GNUM)
@@ -359,10 +458,18 @@ typedef cs_real_33_t  cs_real_332_t[2];  /* vector of 2 3x3 matrices
 # define CS_GNUM_TYPE     CS_UINT32
 #endif
 
-#if (SIZEOF_INT == 8)
-# define CS_LNUM_TYPE     CS_INT64
+#if defined(HAVE_LONG_LNUM)
+# if (SIZEOF_LONG == 8)
+#  define CS_LNUM_TYPE     CS_INT64
+# else
+#  define CS_LNUM_TYPE     CS_INT32
+# endif
 #else
-# define CS_LNUM_TYPE     CS_INT32
+# if (SIZEOF_INT == 8)
+#  define CS_LNUM_TYPE     CS_INT64
+# else
+#  define CS_LNUM_TYPE     CS_INT32
+# endif
 #endif
 
 #if (SIZEOF_INT == 8)
@@ -371,26 +478,35 @@ typedef cs_real_33_t  cs_real_332_t[2];  /* vector of 2 3x3 matrices
 # define CS_INT_TYPE      CS_INT32
 #endif
 
+#if (SIZEOF_INT == 8)
+# define CS_UINT_TYPE     CS_UINT64
+#else
+# define CS_UINT_TYPE     CS_UINT32
+#endif
+
+#define CS_FLAG_TYPE      CS_UINT16
+#define CS_EFLAG_TYPE     CS_UINT_TYPE
 #define CS_REAL_TYPE      CS_DOUBLE
 #define CS_COORD_TYPE     CS_DOUBLE
 
 /* Minimum size for OpenMP loops
- *  (based on initial benchmarking, which will need updates)
- *-----------------------------------------------------------*/
+ *  (will need benchmarking and tuning for various systems)
+ *---------------------------------------------------------*/
 
-#if defined(__bgq__) && defined(__xlc__)
-#  define CS_THR_MIN 1014
-#else
-#  define CS_THR_MIN 128
-#endif
+#define CS_THR_MIN 128
+
+/* Cache line size, or multiple thereof */
+/*--------------------------------------*/
+
+#define CS_CL_SIZE 64
 
 /*----------------------------------------------------------------------------
  * Type independent min an max (caution: the argument is evaluated)
  *----------------------------------------------------------------------------*/
 
-#define CS_ABS(a)     ((a) <  0  ? -(a) : (a))  /* Absolute value of a */
-#define CS_MIN(a,b)   ((a) < (b) ?  (a) : (b))  /* Minimum of a et b */
-#define CS_MAX(a,b)   ((a) > (b) ?  (a) : (b))  /* Maximum of a et b */
+#define CS_ABS(a)     ((a) <  0  ? -(a) : (a))  /*!< Absolute value of a */
+#define CS_MIN(a,b)   ((a) < (b) ?  (a) : (b))  /*!< Minimum of a et b */
+#define CS_MAX(a,b)   ((a) > (b) ?  (a) : (b))  /*!< Maximum of a et b */
 
 /*----------------------------------------------------------------------------
  * Variable interlace type:
@@ -405,6 +521,16 @@ typedef enum {
 
 } cs_interlace_t;
 
+/*----------------------------------------------------------------------------
+ * Macro used to silence "unused argument" warnings.
+ *
+ * This is useful when a function must match a given function pointer
+ * type, but does not use all possible arguments.
+ *----------------------------------------------------------------------------*/
+
+#define CS_UNUSED(x) (void)(x)
+#define CS_NO_WARN_IF_UNUSED(x) (void)(x)
+
 /*----------------------------------------------------------------------------
  * Macros for compilation with a C++ compiler
  *----------------------------------------------------------------------------*/
@@ -421,14 +547,24 @@ typedef enum {
 #endif
 
 /*----------------------------------------------------------------------------
- * Macros for scoping of examples
+ * Macros for function type qualifiers
  *----------------------------------------------------------------------------*/
 
-#undef BEGIN_EXAMPLE_SCOPE
-#undef END_EXAMPLE_SCOPE
+#ifdef __NVCC__
 
-#define BEGIN_EXAMPLE_SCOPE  {
-#define   END_EXAMPLE_SCOPE  }
+#define CS_F_HOST __host__
+#define CS_F_DEVICE __device__
+#define CS_F_HOST_DEVICE __host__ __device__
+#define CS_V_CONSTANT __constant__
+
+#else
+
+#define CS_F_HOST
+#define CS_F_DEVICE
+#define CS_F_HOST_DEVICE
+#define CS_V_CONSTANT static const
+
+#endif
 
 /*----------------------------------------------------------------------------
  * Macros for Fortran interoperability
@@ -445,36 +581,20 @@ typedef enum {
 #define CS_PROCF(x, y) x
 #endif
 
-/*
- * Macro used to handle automatic "Fortran string length" arguments
- * (not used by Code_Saturne calls, but set by many compilers).
- * Some compilers, like the Fujitsu VPP 5000 compiler in its time, may not
- * support the variable length lists in mixed C/Fortran calls.
- */
-
-#if defined (__uxpv__)  /* Fujitsu VPP 5000 case */
-#define CS_ARGF_SUPP_CHAINE
-#else
-#define CS_ARGF_SUPP_CHAINE , ...
-#endif
-
 /*=============================================================================
  * Global variables
  *============================================================================*/
 
-/* Numerical constants */
+/* Empty but non-null string */
 
-extern const cs_real_t cs_defs_epzero;
-extern const cs_real_t cs_defs_infinite_r;
-extern const cs_real_t cs_defs_big_r;
-extern const cs_real_t cs_defs_pi;
+extern const char     cs_empty_string[];
 
 /* Sizes and names associated with datatypes */
 
 extern const size_t   cs_datatype_size[];
 extern const char    *cs_datatype_name[];
 
-/* MPI Datatypes associated with Code_Saturne datatypes */
+/* MPI Datatypes associated with code_saturne datatypes */
 
 #if defined(HAVE_MPI) && !defined(CS_IGNORE_MPI)
 
@@ -482,12 +602,16 @@ extern MPI_Datatype   cs_datatype_to_mpi[];
 
 #endif
 
-/* Global variables indicationg task state */
+/* Global variables indicating task state */
 
-extern int  cs_glob_n_threads;      /* Number of threads */
+extern int  cs_glob_n_threads;     /* Number of threads */
 
-extern int  cs_glob_rank_id;        /* Rank in main MPI communicator */
-extern int  cs_glob_n_ranks;        /* Size of main MPI communicator */
+extern int  cs_glob_rank_id;       /* Rank in main MPI communicator */
+extern int  cs_glob_n_ranks;       /* Size of main MPI communicator */
+
+extern int  cs_glob_node_rank_id;  /* Rank on node in main MPI communicator */
+extern int  cs_glob_node_n_ranks;  /* Number of ranks on node of main
+                                      MPI communicator */
 
 #if defined(HAVE_MPI) && !defined(CS_IGNORE_MPI)
 
@@ -495,10 +619,173 @@ extern MPI_Comm       cs_glob_mpi_comm;      /* Main MPI intra-communicator */
 
 #endif
 
+/*----------------------------------------------------------------------------
+ * Function pointer types
+ *----------------------------------------------------------------------------*/
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Destroy a given structure
+ *
+ * \param[in, out]  s  pointer to a structure cast on-the-fly
+ *
+ * \return a null pointer
+ */
+/*----------------------------------------------------------------------------*/
+
+typedef void *
+(cs_destructor_t)(void  *s);
+
+/*=============================================================================
+ * Public functions
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------*/
+/*
+ * \brief  Given a base index i, return the next index aligned with a size m.
+ *
+ * \param[in]  i   base index
+ * \param[in]  m   block size to align with
+ *
+ * \return aligned index
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static cs_lnum_t
+cs_align(cs_lnum_t  i,
+         cs_lnum_t  m)
+{
+  return ((i > 0) ? ((i-1)/m+1)*m : 0);
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Retrieve the associated thread id (0 if no OpenMP or if outside an
+ *        OpenMP region)
+ *
+ * \return the id of the OpenMP thread
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static int
+cs_get_thread_id(void)
+{
+#if defined(HAVE_OPENMP)
+  return omp_get_thread_num();
+#else
+  return 0;
+#endif
+}
+
 /*----------------------------------------------------------------------------*/
 
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
 
+#ifdef __cplusplus
+
+/*=============================================================================
+ * Public C++ templates
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Get the cs_datatype_t from a typename
+ *
+ * \tparam T : datatype
+ */
+/*----------------------------------------------------------------------------*/
+
+template <typename T>
+struct dependent_false : std::false_type {};
+
+template <typename T>
+static inline cs_datatype_t
+cs_datatype_from_type()
+{
+  static_assert(dependent_false<T>::value, "Unknown datatype");
+  return CS_DATATYPE_NULL;
+}
+
+/*----------------------------------------------------------------------------*/
+/* Specialized versions of the templated function                             */
+/*----------------------------------------------------------------------------*/
+
+template <>
+constexpr inline cs_datatype_t
+cs_datatype_from_type<char>()
+{
+  return CS_CHAR;
+}
+
+/*----------------------------------------------------------------------------*/
+
+template <>
+constexpr inline cs_datatype_t
+cs_datatype_from_type<float>()
+{
+  return CS_FLOAT;
+}
+
+/*----------------------------------------------------------------------------*/
+
+template <>
+constexpr inline cs_datatype_t
+cs_datatype_from_type<double>()
+{
+  return CS_DOUBLE;
+}
+
+/*----------------------------------------------------------------------------*/
+
+template <>
+constexpr inline cs_datatype_t
+cs_datatype_from_type<uint16_t>()
+{
+  return CS_UINT16;
+}
+
+/*----------------------------------------------------------------------------*/
+
+template <>
+constexpr inline cs_datatype_t
+cs_datatype_from_type<uint32_t>()
+{
+  return CS_UINT32;
+}
+
+/*----------------------------------------------------------------------------*/
+
+template <>
+constexpr inline cs_datatype_t
+cs_datatype_from_type<uint64_t>()
+{
+  return CS_UINT64;
+}
+
+/*----------------------------------------------------------------------------*/
+
+template <>
+constexpr inline cs_datatype_t
+cs_datatype_from_type<int32_t>()
+{
+  return CS_INT32;
+}
+
+/*----------------------------------------------------------------------------*/
+
+template <>
+constexpr inline cs_datatype_t
+cs_datatype_from_type<int64_t>()
+{
+  return CS_INT64;
+}
+
+/*----------------------------------------------------------------------------*/
+
+#endif /* __cplusplus */
+
+/*----------------------------------------------------------------------------*/
+
 #endif /* __CS_DEFS_H__ */
