@@ -6,9 +6,9 @@
  *============================================================================*/
 
 /*
-  This file is part of Code_Saturne, a general-purpose CFD tool.
+  This file is part of code_saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2025 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -31,7 +31,7 @@
  *  Local headers
  *----------------------------------------------------------------------------*/
 
-#include "cs_defs.h"
+#include "base/cs_defs.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -41,22 +41,44 @@ BEGIN_C_DECLS
  * Macro definitions
  *============================================================================*/
 
+/*!
+ * @defgroup field_flags Flags specifying general field attributes
+ *
+ * @{
+ */
+
 /*
  * Field property type
  */
 
+/*! represents an intensive quantity */
 #define CS_FIELD_INTENSIVE           (1 << 0)
+
+/*! represents an extensive quantity */
 #define CS_FIELD_EXTENSIVE           (1 << 1)
 
 /* Field category */
 
+/*! resolved variable */
 #define CS_FIELD_VARIABLE            (1 << 2)
+
+/*! computed property */
 #define CS_FIELD_PROPERTY            (1 << 3)
+
+/*! intended for postprocessing */
 #define CS_FIELD_POSTPROCESS         (1 << 4)
+
+/*! accumulator for some statisicial values */
 #define CS_FIELD_ACCUMULATOR         (1 << 5)
 
+/*! user-defined */
 #define CS_FIELD_USER                (1 << 6)
 
+/*! field associated to a CDO equation */
+#define CS_FIELD_CDO                 (1 << 7)
+
+/*! @} */
+
 /*============================================================================
  * Type definitions
  *============================================================================*/
@@ -71,17 +93,40 @@ typedef enum {
   CS_FIELD_INVALID_KEY_ID,
   CS_FIELD_INVALID_CATEGORY,
   CS_FIELD_INVALID_TYPE,
+  CS_FIELD_INVALID_FIELD,
   CS_FIELD_LOCKED
 
 } cs_field_error_type_t;
 
-/* Field boundary condition descriptor (for variables) */
-/*-----------------------------------------------------*/
+/*! boundary condition descriptor when solving in increment */
+/*----------------------------------------------------------*/
+
+typedef struct {
+
+  cs_real_t          *val_ip;      /* variable computed at I' position */
+
+  cs_real_t          *val_f;       /* face value for gradient */
+  cs_real_t          *val_f_lim;   /* face value for gradient
+                                      (limited) */
+
+  cs_real_t          *val_f_d;     /* face value for diffusion */
+  cs_real_t          *val_f_d_lim; /* face value for diffusion
+                                      (limited) */
+
+} cs_bc_coeffs_solve_t;
+
+/*! Field boundary condition descriptor (for variables) */
+/*------------------------------------------------------*/
 
 typedef struct {
 
   int                location_id;  /* Id of matching location */
 
+  int               *icodcl;       /* low-level BC type code */
+  cs_real_t         *rcodcl1;      /* 1st component of low-level BC */
+  cs_real_t         *rcodcl2;      /* 2nd component of low-level BC */
+  cs_real_t         *rcodcl3;      /* 3rd component of low-level BC */
+
   cs_real_t         *a;            /* Explicit coefficient */
   cs_real_t         *b;            /* Implicit coefficient */
   cs_real_t         *af;           /* Explicit coefficient for flux */
@@ -91,6 +136,20 @@ typedef struct {
   cs_real_t         *ac;           /* Explicit coefficient for convection */
   cs_real_t         *bc;           /* Implicit coefficient for convection */
 
+  cs_real_t         *hint;         /* coefficient for internal coupling */
+  cs_real_t         *_hext;        /* coefficient for internal coupling
+                                      (maps to rcodcl2 when present) */
+
+  cs_real_t          *val_f;       /* face value for gradient */
+  cs_real_t          *val_f_lim;   /* face value for gradient
+                                      (limited) */
+
+  cs_real_t          *val_f_d;     /* face value for diffusion */
+  cs_real_t          *val_f_d_lim; /* face value for diffusion
+                                      (limited) */
+
+  cs_real_t          *val_f_pre;   /* previous face value for gradient */
+
 } cs_field_bc_coeffs_t;
 
 /* Field descriptor */
@@ -104,7 +163,6 @@ typedef struct {
   int                     type;         /* Field type flag */
 
   int                     dim;          /* Field dimension */
-  bool                    interleaved;  /* is field interleaved ? */
 
   int                     location_id;  /* Id of matching location */
 
@@ -125,6 +183,10 @@ typedef struct {
                                            to matching previous values array
                                            (if n_time_vals == 2) */
 
+  cs_real_t              *grad;         /* For each active location, pointer
+                                           to matching gradient array for
+                                           variable type fields */
+
   cs_field_bc_coeffs_t   *bc_coeffs;    /* Boundary condition coefficients,
                                            for variable type fields */
 
@@ -142,6 +204,16 @@ typedef struct {
 typedef void
 (cs_field_log_key_struct_t) (const void  *t);
 
+/*----------------------------------------------------------------------------
+ * Function pointer for structure associated to field key
+ *
+ * parameters:
+ *   t <-- pointer to structure
+ *----------------------------------------------------------------------------*/
+
+typedef void
+(cs_field_clear_key_struct_t) (void  *t);
+
 /*============================================================================
  * Global variables
  *============================================================================*/
@@ -156,6 +228,37 @@ extern const char *cs_glob_field_comp_name_9[];
  * Public function prototypes
  *============================================================================*/
 
+/*----------------------------------------------------------------------------
+ * Create bc_coeffs structure.
+ *
+ * \param[in]     bc_coeffs   bc coefficients
+ *----------------------------------------------------------------------------*/
+
+void
+cs_field_bc_coeffs_init(cs_field_bc_coeffs_t  *bc_coeffs);
+
+/*----------------------------------------------------------------------------
+ * Shallow copy of boundary condition coefficients.
+ *
+ * \param[in]     ref   source bc coefficients
+ * \param[inout]  copy  shallow copy of bc coefficients
+ *----------------------------------------------------------------------------*/
+
+void
+cs_field_bc_coeffs_shallow_copy(const cs_field_bc_coeffs_t  *ref,
+                                cs_field_bc_coeffs_t        *copy);
+
+/*----------------------------------------------------------------------------
+ * \brief  Free copy of boundary condition coefficients.
+ *
+ * \param[in]     ref   reference bc coefficients
+ * \param[inout]  copy  shallow copy of bc coefficients
+ *----------------------------------------------------------------------------*/
+
+void
+cs_field_bc_coeffs_free_copy(const cs_field_bc_coeffs_t  *ref,
+                             cs_field_bc_coeffs_t        *copy);
+
 /*----------------------------------------------------------------------------
  * Return the number of defined fields.
  *
@@ -174,8 +277,6 @@ cs_field_n_fields(void);
  *   type_flag    <-- mask of field property and category values
  *   location_id  <-- id of associated location
  *   dim          <-- field dimension (number of components)
- *   interleaved  <-- indicate if values are interleaved
- *                    (ignored if number of components < 2)
  *   has_previous <-- maintain values at the previous time step ?
  *
  * returns:
@@ -187,37 +288,57 @@ cs_field_create(const char   *name,
                 int           type_flag,
                 int           location_id,
                 int           dim,
-                bool          interleaved,
                 bool          has_previous);
 
-/*----------------------------------------------------------------------------
- * Return a field matching a given name and attributes,
- * creating it if necessary.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create a field descriptor.
+ *
+ * \param[in]  name_prefix   first part of field name
+ * \param[in]  name_suffix   second part of field name
+ * \param[in]  location_id   id of associated location
+ * \param[in]  dim           field dimension (number of components)
+ * \param[in]  has_previous  maintain values at the previous time step ?
+ *
+ * \return  pointer to new field.
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_field_t *
+cs_field_create_by_composite_name(const char   *name_prefix,
+                                  const char   *name_suffix,
+                                  int           type_flag,
+                                  int           location_id,
+                                  int           dim,
+                                  bool          has_previous);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Return a field matching a given name and attributes,
+ *        creating it if necessary.
  *
  * If a field with the same name but different attributes is present,
  * this is considered an error.
  *
  * The default number of time values associated with a field created through
- * this function is 1. To modify it, use cs_field_set_n_time_vals().
+ * this function is 1. To modify it, use \ref cs_field_set_n_time_vals.
  *
- * parameters:
- *   name        <-- field name
- *   type_flag   <-- mask of field property and category values
- *   location_id <-- id of associated location
- *   dim         <-- field dimension (number of components)
- *   interleaved <-- indicate if values ar interleaved
- *                   (ignored if number of components < 2)
+ * \param[in]  name          field name
+ * \param[in]  type_flag     mask of field property and category values
+ * \param[in]  location_id   id of associated location
+ * \param[in]  dim           field dimension (number of components)
+ * \param[in]  has_previous  maintain values at the previous time step ?
  *
- * returns:
- *   pointer to field
- *----------------------------------------------------------------------------*/
+ * \return  pointer to field
+ */
+/*----------------------------------------------------------------------------*/
 
 cs_field_t *
 cs_field_find_or_create(const char   *name,
                         int           type_flag,
                         int           location_id,
                         int           dim,
-                        bool          interleaved);
+                        bool          has_previous);
 
 /*----------------------------------------------------------------------------
  * Change the number of time values managed by a field.
@@ -249,7 +370,7 @@ cs_field_allocate_values(cs_field_t  *f);
  * parameters:
  *   f           <-> pointer to field structure
  *   val         <-- pointer to array of values
- *   val_pre     <-- pointer to array of previous values, or NULL
+ *   val_pre     <-- pointer to array of previous values, or nullptr
  *----------------------------------------------------------------------------*/
 
 void
@@ -267,26 +388,25 @@ cs_field_map_values(cs_field_t   *f,
  * locations (though support could be added by mapping a boundary->location
  * indirection array in the cs_mesh_location_t structure).
  *
- * For multidimensional fields, arrays are assumed to have the same
- * interleaving behavior as the field, unless components are coupled.
- *
- * For multidimensional fields with coupled components, interleaving
- * is the norm, and implicit b and bf coefficient arrays are arrays of
- * block matrices, not vectors, so the number of entries for each boundary
- * face is dim*dim instead of dim.
+ * For multidimensional fields with coupled components, implicit b and bf
+ * coefficient arrays are arrays of block matrices, not vectors, so the
+ * number of entries for each boundary face is dim*dim instead of dim.
  *
  * parameters:
  *   f            <-- pointer to field structure
  *   have_flux_bc <-- if true, flux BC coefficients (af and bf) are added
  *   have_mom_bc  <-- if true, div BC coefficients (ad and bd) are added
  *   have_conv_bc <-- if true, convection BC coefficients (ac and bc) are added
+ *   have_exch_bc <-- if true, exchange boundary coefficients (hint and hext)
+ *                    are added
  *----------------------------------------------------------------------------*/
 
 void
 cs_field_allocate_bc_coeffs(cs_field_t  *f,
                             bool         have_flux_bc,
                             bool         have_mom_bc,
-                            bool         have_conv_bc);
+                            bool         have_conv_bc,
+                            bool         have_exch_bc);
 
 /*----------------------------------------------------------------------------*/
 /* Initialize boundary condition coefficients arrays.
@@ -298,13 +418,9 @@ cs_field_allocate_bc_coeffs(cs_field_t  *f,
  * locations (though support could be added by mapping a boundary->location
  * indirection array in the cs_mesh_location_t structure).
  *
- * For multidimensional fields, arrays are assumed to have the same
- * interleaving behavior as the field, unless components are coupled.
- *
- * For multidimensional fields with coupled components, interleaving
- * is the norm, and implicit b and bf coefficient arrays are arrays of
- * block matrices, not vectors, so the number of entries for each boundary
- * face is dim*dim instead of dim.
+ * For multidimensional fields with coupled components, implicit b and bf
+ * coefficient arrays are arrays of block matrices, not vectors, so the
+ * number of entries for each boundary face is dim*dim instead of dim.
  *
  * parameters:
  *   f <-> pointer to field structure
@@ -313,6 +429,28 @@ cs_field_allocate_bc_coeffs(cs_field_t  *f,
 void
 cs_field_init_bc_coeffs(cs_field_t  *f);
 
+/*----------------------------------------------------------------------------
+ * Allocate arrays for field gradient.
+ *
+ * parameters:
+ *   f <-- pointer to field structure
+ *----------------------------------------------------------------------------*/
+
+void
+cs_field_allocate_gradient(cs_field_t  *f);
+
+/*----------------------------------------------------------------------------
+ * Set current field values to the given constant.
+ *
+ * parameters:
+ *   f <-> pointer to field structure
+ *   c <-- assigned value
+ *----------------------------------------------------------------------------*/
+
+void
+cs_field_set_values(cs_field_t  *f,
+                    cs_real_t    c);
+
 /*----------------------------------------------------------------------------
  * Copy current field values to previous values if applicable.
  *
@@ -378,18 +516,82 @@ cs_field_by_name(const char *name);
 /*----------------------------------------------------------------------------
  * Return a pointer to a field based on its name if present.
  *
- * If no field of the given name is defined, NULL is returned.
+ * If no field of the given name is defined, nullptr is returned.
  *
  * parameters:
  *   name <-- field name
  *
  * returns:
- *   pointer to the field structure, or NULL
+ *   pointer to the field structure, or nullptr
  *----------------------------------------------------------------------------*/
 
 cs_field_t  *
 cs_field_by_name_try(const char *name);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Return a pointer to a field based on a composite name.
+ *
+ * The name is expected to be of the form <name_prefix>_<name_suffix>.
+ *
+ * \param[in]  name_prefix  first part of field name
+ * \param[in]  name_suffix  second part of field name
+ *
+ * \return  pointer to the field structure, or nullptr
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_field_t  *
+cs_field_by_composite_name(const char  *name_prefix,
+                           const char  *name_suffix);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Return a pointer to a field based on a composite name if present.
+ *
+ * The name is expected to be of the form <name_prefix>_<name_suffix>.
+ * If no field of the given name is defined, nullptr is returned.
+ *
+ * \remark: in C++, we could simply have a cs_field_by_name_try template
+ *          with a variable number of arguments.
+ *
+ * \param[in]  name_prefix  first part of field name
+ * \param[in]  name_suffix  second part of field name
+ *
+ * \return  pointer to the field structure, or nullptr
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_field_t  *
+cs_field_by_composite_name_try(const char  *name_prefix,
+                               const char  *name_suffix);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Return pointer to a field based on a double composite name if present.
+ *
+ * If no field of the given name is defined, a null pointer is returned.
+ *
+ * Contrary to \ref cs_field_by_composite_name_try, this function
+ * does not automatically add '_' characters between component names.
+ * This allows adding different separators, such as ':'.
+
+ * \remark: in C++, we could simply have a cs_field_by_name_try template
+ *          with a variable number of arguments.
+ *
+ * \param[in]  name_part_1  first part of field name
+ * \param[in]  name_part_2  second part of field name
+ * \param[in]  name_part_3  second part of field name
+ *
+ * \return  pointer to the field structure, or nullptr
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_field_t  *
+cs_field_by_double_composite_name_try(const char  *name_part_1,
+                                      const char  *name_part_2,
+                                      const char  *name_part_3);
+
 /*----------------------------------------------------------------------------
  * Return the id of a defined field based on its name.
  *
@@ -525,23 +727,27 @@ cs_field_define_key_str(const char  *name,
  * by the current value, and its id is returned.
  *
  * parameters:
- *   name          <-- key name
- *   default_value <-- pointer to default value associated with key
- *   log_funct     <-- pointer to logging function
- *   size          <-- sizeof structure
- *   type_flag     <-- mask associated with field types with which
- *                     the key may be associated, or 0
+ *   name             <-- key name
+ *   default_value    <-- pointer to default value associated with key
+ *   log_funct        <-- pointer to logging function
+ *   log_func_default <-- pointer to default logging function
+ *   clear_func       <-- pointer to substructures free function
+ *   size             <-- sizeof structure
+ *   type_flag        <-- mask associated with field types with which
+ *                        the key may be associated, or 0
  *
  * returns:
  *   id associated with key
  *----------------------------------------------------------------------------*/
 
 int
-cs_field_define_key_struct(const char                 *name,
-                           const void                 *default_value,
-                           cs_field_log_key_struct_t  *log_func,
-                           size_t                      size,
-                           int                         type_flag);
+cs_field_define_key_struct(const char                   *name,
+                           const void                   *default_value,
+                           cs_field_log_key_struct_t    *log_func,
+                           cs_field_log_key_struct_t    *log_func_default,
+                           cs_field_clear_key_struct_t  *clear_func,
+                           size_t                        size,
+                           int                           type_flag);
 
 /*----------------------------------------------------------------------------
  * Define a sub key.
@@ -695,6 +901,52 @@ int
 cs_field_get_key_int(const cs_field_t  *f,
                      int                key_id);
 
+/*----------------------------------------------------------------------------
+ * Set integer bits matching a mask to 1 for a given key for a field.
+ *
+ * If the key id is not valid, CS_FIELD_INVALID_KEY_ID is returned.
+ * If the field category is not compatible with the key (as defined
+ * by its type flag), CS_FIELD_INVALID_CATEGORY is returned.
+ * If the data type does not match, CS_FIELD_INVALID_TYPE is returned.
+ * If the key value has been locked, CS_FIELD_LOCKED is returned.
+ *
+ * parameters:
+ *   f             <-- pointer to field structure
+ *   key_id        <-- id of associated key
+ *   mask          <-- mask associated with key
+ *
+ * returns:
+ *   0 in case of success, > 1 in case of error
+ *----------------------------------------------------------------------------*/
+
+int
+cs_field_set_key_int_bits(cs_field_t  *f,
+                          int          key_id,
+                          int          mask);
+
+/*----------------------------------------------------------------------------
+ * Set integer bits matching a mask to 0 for a given key for a field.
+ *
+ * If the key id is not valid, CS_FIELD_INVALID_KEY_ID is returned.
+ * If the field category is not compatible with the key (as defined
+ * by its type flag), CS_FIELD_INVALID_CATEGORY is returned.
+ * If the data type does not match, CS_FIELD_INVALID_TYPE is returned.
+ * If the key value has been locked, CS_FIELD_LOCKED is returned.
+ *
+ * parameters:
+ *   f             <-- pointer to field structure
+ *   key_id        <-- id of associated key
+ *   mask          <-- mask associated with key
+ *
+ * returns:
+ *   0 in case of success, > 1 in case of error
+ *----------------------------------------------------------------------------*/
+
+int
+cs_field_clear_key_int_bits(cs_field_t  *f,
+                            int          key_id,
+                            int          mask);
+
 /*----------------------------------------------------------------------------
  * Assign a floating point value for a given key to a field.
  *
@@ -821,6 +1073,49 @@ cs_field_get_key_struct(const cs_field_t  *f,
                         int                key_id,
                         void              *s);
 
+/*----------------------------------------------------------------------------*/
+/*
+ * \brief Return a pointer to a simple structure for a given key to a field.
+ *
+ * If the key id is not valid, the value type or field category is not
+ * compatible, or the structure has been locked, a fatal error is provoked.
+ *
+ * Note that using this function marks the field's value for this structure
+ * as set, and if no values have been set yet, the structure is set to
+ * default values.
+ *
+ * \param[in]  f       pointer to field structure
+ * \param[in]  key_id  id of associated key
+ *
+ * \return  pointer to key structure in case of success,
+ *          nullptr in case of error
+ */
+/*----------------------------------------------------------------------------*/
+
+void *
+cs_field_get_key_struct_ptr(cs_field_t  *f,
+                            int          key_id);
+
+/*----------------------------------------------------------------------------*/
+/*
+ * \brief Return a read-only pointer to a simple structure for a given key
+ *        to a field.
+ *
+ * If the key id is not valid, the value type or field category is not
+ * compatible, a fatal error is provoked.
+ *
+ * \param[in]  f       pointer to field structure
+ * \param[in]  key_id  id of associated key
+ *
+ * \return  pointer to key structure in case of success,
+ *          nullptr in case of error
+ */
+/*----------------------------------------------------------------------------*/
+
+const void *
+cs_field_get_key_struct_const_ptr(const cs_field_t  *f,
+                                  int                key_id);
+
 /*----------------------------------------------------------------------------
  * Print info relative to all field definitions to log file.
  *----------------------------------------------------------------------------*/
@@ -890,7 +1185,6 @@ cs_field_log_all_key_vals(bool  log_defaults);
  *   "label"        (string)
  *   "log"          (integer)
  *   "post_vis"     (integer)
- *   "post_probes"  (integer)
  *   "coupled"      (integer, restricted to CS_FIELD_VARIABLE)
  *   "moment_id"    (integer, restricted to
  *                   CS_FIELD_ACCUMULATOR | CS_FIELD_POSTPROCESS);
