@@ -6,9 +6,9 @@
  *============================================================================*/
 
 /*
-  This file is part of Code_Saturne, a general-purpose CFD tool.
+  This file is part of code_saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2025 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -27,15 +27,16 @@
 
 /*----------------------------------------------------------------------------*/
 
-#include "cs_defs.h"
+#include "base/cs_defs.h"
 
 /*----------------------------------------------------------------------------
  *  Local headers
  *----------------------------------------------------------------------------*/
 
-#include "fvm_defs.h"
-#include "fvm_group.h"
-#include "fvm_io_num.h"
+#include "mesh/cs_mesh.h"
+#include "fvm/fvm_defs.h"
+#include "fvm/fvm_group.h"
+#include "fvm/fvm_io_num.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -89,7 +90,7 @@ fvm_nodal_create(const char  *name,
  *   this_nodal  <-> pointer to structure that should be destroyed
  *
  * returns:
- *  NULL pointer
+ *  null pointer
  *----------------------------------------------------------------------------*/
 
 fvm_nodal_t *
@@ -133,25 +134,25 @@ fvm_nodal_reduce(fvm_nodal_t  *this_nodal,
  * structure's creation.
  *
  * parameters:
- *   this_nodal          <-- nodal mesh structure
- *   new_parent_num      <-- pointer to local parent renumbering array
- *                           ({1, ..., n} <-- {1, ..., n})
- *   entity_dim          <-- 3 for cells, 2 for faces, 1 for edges,
- *                           and 0 for vertices
+ *   this_nodal         <-- nodal mesh structure
+ *   new_parent_id      <-- pointer to local parent renumbering array
+ *                          ({0, ..., n-1} <-- {0, ..., n-1})
+ *   entity_dim         <-- 3 for cells, 2 for faces, 1 for edges,
+ *                          and 0 for vertices
  *----------------------------------------------------------------------------*/
 
 void
-fvm_nodal_change_parent_num(fvm_nodal_t        *this_nodal,
-                            const cs_lnum_t     new_parent_num[],
-                            int                 entity_dim);
+fvm_nodal_change_parent_id(fvm_nodal_t       *this_nodal,
+                           const cs_lnum_t    new_parent_id[],
+                           int                entity_dim);
 
 /*----------------------------------------------------------------------------
  * Remove entity parent numbering; this is useful for example when we
  * want to assign coordinates or fields to an extracted mesh using
  * arrays relative to the mesh, and not to its parent.
  *
- * This is equivalent to calling fvm_nodal_change_parent_num(), with
- * 'trivial' (1 o n) new_parent_num[] values.
+ * This is equivalent to calling fvm_nodal_change_parent_id(), with
+ * 'trivial' (1 o n) new_parent_id[] values.
  *
  * parameters:
  *   this_nodal          <-- nodal mesh structure
@@ -160,8 +161,8 @@ fvm_nodal_change_parent_num(fvm_nodal_t        *this_nodal,
  *----------------------------------------------------------------------------*/
 
 void
-fvm_nodal_remove_parent_num(fvm_nodal_t  *this_nodal,
-                            int           entity_dim);
+fvm_nodal_remove_parent_id(fvm_nodal_t  *this_nodal,
+                           int           entity_dim);
 
 /*----------------------------------------------------------------------------
  * Build external numbering for entities based on global numbers.
@@ -179,12 +180,65 @@ fvm_nodal_init_io_num(fvm_nodal_t        *this_nodal,
                       const cs_gnum_t     parent_global_numbers[],
                       int                 entity_dim);
 
+/*----------------------------------------------------------------------------
+ * Transfer an existing global numbering structure to a nodal mesh.
+ *
+ * This assumes the local number of vertices is identical to that of
+ * the numberign structure.
+ *
+ * The argument pointer to the global numbering structure is set to NULL
+ * so the caller does not spuriously modify the structure after this call.
+ *
+ * A call to this function may be used instead of fvm_nodal_init_io_num
+ * (for vertices only).
+ *
+ * parameters:
+ *   this_nodal    <-- nodal mesh structure
+ *   io_num        <-> pointer to external numbering structure
+ *                     whose property is transferred to the mesh
+ *                     independent) parent entity numbers
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_transfer_vertex_io_num(fvm_nodal_t    *this_nodal,
+                                 fvm_io_num_t  **io_num);
+
+/*----------------------------------------------------------------------------
+ * Set entity tags (for non-vertex entities).
+ *
+ * The number of entities of the given dimension may be obtained
+ * through fvm_nodal_get_n_entities(), the tag[] array is populated
+ * in local section order, section by section).
+ *
+ * parameters:
+ *   this_nodal <-- nodal mesh structure
+ *   tag        <-- tag values to assign
+ *   entity_dim <-- 3 for cells, 2 for faces, 1 for edges
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_set_tag(fvm_nodal_t  *this_nodal,
+                  const int     tag[],
+                  int           entity_dim);
+
+/*----------------------------------------------------------------------------
+ * Remove entity tags.
+ *
+ * parameters:
+ *   this_nodal <-- nodal mesh structure
+ *   entity_dim <-- 3 for cells, 2 for faces, 1 for edges
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_remove_tag(fvm_nodal_t  *this_nodal,
+                     int           entity_dim);
+
 /*----------------------------------------------------------------------------
  * Preset number and list of vertices to assign to a nodal mesh.
  *
- * If the parent_vertex_num argument is NULL, the list is assumed to
- * be {1, 2, ..., n}. If parent_vertex_num is given, it specifies a
- * list of n vertices from a larger set (1 to n numbering).
+ * If the parent_vertex_id argument is NULL, the list is assumed to
+ * be {0, 1, ..., n-1}. If parent_vertex_id is given, it specifies a
+ * list of n vertices from a larger set (0 to n-1 numbering).
  *
  * Ownership of the given parent vertex numbering array is
  * transferred to the nodal mesh representation structure.
@@ -195,15 +249,15 @@ fvm_nodal_init_io_num(fvm_nodal_t        *this_nodal,
  * a mesh containing only vertices).
  *
  * parameters:
- *   this_nodal        <-> nodal mesh structure
- *   n_vertices        <-- number of vertices to assign
- *   parent_vertex_num <-- parent numbers of vertices to assign
+ *   this_nodal       <-> nodal mesh structure
+ *   n_vertices       <-- number of vertices to assign
+ *   parent_vertex_id <-- parent ids of vertices to assign
  *----------------------------------------------------------------------------*/
 
 void
 fvm_nodal_define_vertex_list(fvm_nodal_t  *this_nodal,
                              cs_lnum_t     n_vertices,
-                             cs_lnum_t     parent_vertex_num[]);
+                             cs_lnum_t     parent_vertex_id[]);
 
 /*----------------------------------------------------------------------------
  * Assign shared vertex coordinates to an extracted nodal mesh,
@@ -250,7 +304,7 @@ fvm_nodal_transfer_vertices(fvm_nodal_t  *this_nodal,
  * Make vertex coordinates of a nodal mesh private.
  *
  * If vertex coordinates were previously shared, those coordinates that
- * are actually refernces are copied, and the relation to parent vertices
+ * are actually references are copied, and the relation to parent vertices
  * is discarded.
  *
  * If vertices were already private, the mesh is not modified.
@@ -281,6 +335,30 @@ void
 fvm_nodal_set_group_class_set(fvm_nodal_t                  *this_nodal,
                               const fvm_group_class_set_t  *gc_set);
 
+/*----------------------------------------------------------------------------
+ * Assign global vertex labels to a nodal mesh.
+ *
+ * As these are expected to be used only for small sets (i.e. probes)
+ * where the point set is built from a global definition and data movement
+ * would add complexity and overhead, the labels refer to a global view
+ * on rank 0.
+ *
+ * The size of the labels pointers array should be the same as that
+ * returned by fvm_nodal_n_g_vertices();
+ *
+ * This function should only be called once the nodal mesh representation
+ * has been completed, as most functions modifying its vertex definitions
+ * will remove these labels.
+ *
+ * parameters:
+ *   this_nodal <-> nodal mesh structure
+ *   g_labels   <-- global vertex labels, or NULL
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_transfer_global_vertex_labels(fvm_nodal_t  *this_nodal,
+                                        char         *g_labels[]);
+
 /*----------------------------------------------------------------------------
  * Obtain the name of a nodal mesh.
  *
@@ -387,6 +465,9 @@ fvm_nodal_get_n_elements(const fvm_nodal_t  *this_nodal,
  * with the parent entity numbers of those entities, in order (i.e. in
  * local section order, section by section).
  *
+ * This function is similar to fvm_nodal_get_parent_num(), but returns
+ * numbers (1 to n) instead of ids (0 to n-1).
+ *
  * parameters:
  *   this_nodal <-- pointer to nodal mesh structure
  *   entity_dim <-- dimension of entities we are interested in (0 to 3)
@@ -398,6 +479,72 @@ fvm_nodal_get_parent_num(const fvm_nodal_t  *this_nodal,
                          int                 entity_dim,
                          cs_lnum_t           parent_num[]);
 
+/*----------------------------------------------------------------------------
+ * Return local parent id array for all entities of a given
+ * dimension in a nodal mesh.
+ *
+ * The number of entities of the given dimension may be obtained
+ * through fvm_nodal_get_n_entities(), the parent_num[] array is populated
+ * with the parent entity numbers of those entities, in order (i.e. in
+ * local section order, section by section).
+ *
+ * parameters:
+ *   this_nodal <-- pointer to nodal mesh structure
+ *   entity_dim <-- dimension of entities we are interested in (0 to 3)
+ *   parent_id --> entity parent id (array must be pre-allocated)
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_get_parent_id(const fvm_nodal_t  *this_nodal,
+                        int                 entity_dim,
+                        cs_lnum_t           parent_id[]);
+
+/*----------------------------------------------------------------------------
+ * Return pointer to global vertex labels of a nodal mesh.
+ *
+ * As these are expected to be used only for small sets (i.e. probes)
+ * where the point set is built from a global definition and data movement
+ * would adds complexity and overhead, the labels refer to a global view
+ * on rank 0; for the same reason, only shared labels are needed.
+ *
+ * The size of the labels pointers array returned should be the same
+ * as that returned by fvm_nodal_n_g_vertices();
+ *
+ * parameters:
+ *   this_nodal <-> nodal mesh structure
+ *
+ * returns:
+ *   pointer to global vertex labels, or NULL
+ *----------------------------------------------------------------------------*/
+
+const char **
+fvm_nodal_get_global_vertex_labels(const fvm_nodal_t  *this_nodal);
+
+/*----------------------------------------------------------------------------
+ * Return a const pointer to a nodal mesh representation's parent mesh.
+ *
+ * parameters:
+ *   this_nodal <-> pointer to structure that should be reduced
+ *
+ * return:
+ *   const pointer to parent mesh
+ *----------------------------------------------------------------------------*/
+
+const cs_mesh_t  *
+fvm_nodal_get_parent(const fvm_nodal_t  *this_nodal);
+
+/*----------------------------------------------------------------------------
+ * Associate a parent mesh to a nodal mesh representation structure.
+ *
+ * parameters:
+ *   this_nodal <-> pointer to structure that should be reduced
+ *   parent     <-- const pointer to parent mesh
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_set_parent(fvm_nodal_t      *this_nodal,
+                     const cs_mesh_t  *parent);
+
 /*----------------------------------------------------------------------------
  * Compute tesselation a a nodal mesh's sections of a given type, and add the
  * corresponding structure to the mesh representation.
