@@ -6,9 +6,9 @@
  *============================================================================*/
 
 /*
-  This file is part of Code_Saturne, a general-purpose CFD tool.
+  This file is part of code_saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2025 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -27,17 +27,17 @@
 
 /*----------------------------------------------------------------------------*/
 
-#include "cs_defs.h"
+#include "base/cs_defs.h"
 
 /*----------------------------------------------------------------------------
  *  Local headers
  *----------------------------------------------------------------------------*/
 
-#include "cs_block_dist.h"
-#include "cs_part_to_block.h"
-#include "fvm_defs.h"
-#include "fvm_nodal.h"
-#include "fvm_writer.h"
+#include "base/cs_block_dist.h"
+#include "base/cs_part_to_block.h"
+#include "fvm/fvm_defs.h"
+#include "fvm/fvm_nodal.h"
+#include "fvm/fvm_writer.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -123,9 +123,12 @@ typedef void
  *
  * parameters:
  *   mesh                 <-- pointer to nodal mesh structure
+ *   min_export_dim       <-- minimum dimension of sections to export
+ *   max_export_dim       <-- maximum dimension of sections to export
+ *   boundary_flag        <-- -1 if ignored, otherwise only sections
+ *                            with -1 or matching boundary flag
  *   group_by_type        <-- if true, group sections of same type
  *   group_all            <-- if true, all sections continue previous ones
- *   min_export_dim       <-- minimum dimension of sections to export
  *   discard_polygons     <-- ignore polygonal sections
  *   discard_polyhedra    <-- ignore polyhedral sections
  *   divide_polygons      <-- tesselate polygonal sections
@@ -139,6 +142,8 @@ typedef void
 fvm_writer_section_t *
 fvm_writer_export_list(const fvm_nodal_t          *mesh,
                        int                         min_export_dim,
+                       int                         max_export_dim,
+                       int                         boundary_flag,
                        bool                        group_by_type,
                        bool                        group_all,
                        bool                        discard_polygons,
@@ -473,6 +478,24 @@ fvm_writer_field_helper_output_n(fvm_writer_field_helper_t  *helper,
                                  const void           *const field_values[],
                                  fvm_writer_field_output_t  *output_func);
 
+/*----------------------------------------------------------------------------
+ * Set string representing a field component's name based on its id.
+ *
+ * parameters:
+ *   s                  --> destination string
+ *   s_size             <-- maximum string size
+ *   lowercase          <-- true if lowercase is required
+ *   dimension          <-- field dimension
+ *   component_id       <-- field component id
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_writer_field_component_name(char    *s,
+                                size_t   s_size,
+                                bool     lowercase,
+                                int      dimension,
+                                int      component_id);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
