@@ -6,9 +6,9 @@
  *===========================================================================*/
 
 /*
-  This file is part of Code_Saturne, a general-purpose CFD tool.
+  This file is part of code_saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2025 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -37,11 +37,12 @@
  * Local headers
  *---------------------------------------------------------------------------*/
 
-#include "fvm_defs.h"
-#include "fvm_periodicity.h"
+#include "fvm/fvm_defs.h"
+#include "fvm/fvm_periodicity.h"
 
-#include "cs_base.h"
-#include "cs_selector.h"
+#include "base/cs_base.h"
+#include "base/cs_selector.h"
+#include "base/cs_timer.h"
 
 /*---------------------------------------------------------------------------*/
 
@@ -71,6 +72,42 @@ typedef enum {
 
 } cs_join_state_t;
 
+/*----------------------------------------------------------------------------
+ * Joining statistics
+ *----------------------------------------------------------------------------*/
+
+typedef struct {
+
+  int        n_calls;               /* number of calls */
+
+  /* Intersection determination info */
+
+  int        bbox_layout;            /* bounding box layout */
+  cs_gnum_t  bbox_depth[3];          /* box tree depth */
+  cs_gnum_t  n_leaves[3];            /* number of leaves */
+  cs_gnum_t  n_boxes[3];             /* number of boxes */
+  cs_gnum_t  n_th_leaves[3];         /* number leaves over threshold */
+  cs_gnum_t  n_leaf_boxes[3];        /* number of boxes per leaf */
+  cs_gnum_t  box_mem_final[3];       /* final box memory required */
+  cs_gnum_t  box_mem_required[3];    /* memory required */
+
+  cs_timer_counter_t  t_box_build;   /* box build times */
+  cs_timer_counter_t  t_box_query;   /* box query times */
+  cs_timer_counter_t  t_inter_sort;   /* sort intersections times */
+
+  /* other info */
+
+  cs_timer_counter_t  t_l_join_mesh;  /* build local joining mesh times */
+  cs_timer_counter_t  t_edge_inter;   /* edge intersection times */
+  cs_timer_counter_t  t_new_vtx;      /* new vertices times */
+  cs_timer_counter_t  t_merge_vtx;    /* merge vertices times */
+  cs_timer_counter_t  t_u_merge_vtx;  /* update after merge vertices times */
+  cs_timer_counter_t  t_split_faces;  /* split faces times */
+
+  cs_timer_counter_t  t_total;        /* total time */
+
+}  cs_join_stats_t;
+
 /*----------------------------------------------------------------------------
  * Set of user parameters to control the join operation
  *----------------------------------------------------------------------------*/
@@ -205,7 +242,7 @@ typedef struct {
                                 for the joining operation */
   cs_gnum_t     n_g_faces;   /* Global number of border faces selected
                                 for the joining operation */
-  cs_lnum_t    *faces;       /* List of selected border faces */
+  cs_lnum_t    *faces;       /* List of selected border faces (1-based) */
 
   cs_gnum_t    *compact_face_gnum;    /* Global face numbering defined
                                          on the selected faces */
@@ -260,6 +297,8 @@ typedef struct {
   cs_join_param_t   param;       /* Set of parameters used to control
                                     the joining operations */
 
+  cs_join_stats_t   stats;       /* Performance statistics */
+
   cs_join_select_t  *selection;  /* Store entities implied in the joining
                                     operation */
 
@@ -330,6 +369,7 @@ cs_join_destroy(cs_join_t  **join);
  *
  * parameters:
  *   selection_criteria <-- pointer to a cs_mesh_select_t structure
+ *   perio_type         <-- periodicity type (FVM_PERIODICITY_NULL if none)
  *   verbosity          <-- level of verbosity required
  *
  * returns:
@@ -337,8 +377,9 @@ cs_join_destroy(cs_join_t  **join);
  *---------------------------------------------------------------------------*/
 
 cs_join_select_t *
-cs_join_select_create(const char  *selection_criteria,
-                      int          verbosity);
+cs_join_select_create(const char              *selection_criteria,
+                      fvm_periodicity_type_t   perio_type,
+                      int                      verbosity);
 
 /*----------------------------------------------------------------------------
  * Destroy a cs_join_select_t structure.
@@ -349,12 +390,15 @@ cs_join_select_create(const char  *selection_criteria,
  *---------------------------------------------------------------------------*/
 
 void
-join_select_destroy(cs_join_param_t     param,
-                    cs_join_select_t  **join_select);
+cs_join_select_destroy(cs_join_param_t     param,
+                       cs_join_select_t  **join_select);
 
 /*----------------------------------------------------------------------------
  * Extract vertices from a selection of faces.
  *
+ * This function uses 1-based selection lists
+ * (select_faces and select_vertices)
+ *
  * parameters:
  *   n_select_faces <-- number of selected faces
  *   select_faces   <-- list of faces selected
